@@ -17,10 +17,10 @@ template<typename Base>
 struct pipeline_builder_i : query_builder_i<Base> {
     pipeline_builder_i(ecs_pipeline_desc_t *desc, int32_t term_index = 0) 
         : query_builder_i<Base>(&desc->query, term_index)
-        , m_desc(desc) { }
+        , desc_(desc) { }
 
 private:
-    ecs_pipeline_desc_t *m_desc;
+    ecs_pipeline_desc_t *desc_;
 };
 
 }
