@@ -5,7 +5,6 @@
 
 #include "meta.h"
 #include <inttypes.h>
-#include <ctype.h>
 
 #ifdef FLECS_META
 #ifdef FLECS_QUERY_DSL
@@ -251,7 +250,9 @@ int ecs_meta_next(
     scope->op_cur += op->op_count;
 
     if (scope->op_cur >= scope->op_count) {
-        ecs_err("out of bounds");
+        char *str = ecs_get_path(cursor->world, scope->type);
+        ecs_err("too many elements for scope for type %s", str);
+        ecs_os_free(str);
         return -1;
     }
 
