@@ -48,11 +48,15 @@ ecs_sparse_page_t* flecs_sparse_page_new(
     /* Initialize the data array with zero's to guarantee that data is 
      * always initialized. When an entry is removed, data is reset back to
      * zero. Initialize now, as this can take advantage of calloc. */
-    result->data = a ? flecs_calloc(a, sparse->size * FLECS_SPARSE_PAGE_SIZE)
-                     : ecs_os_calloc(sparse->size * FLECS_SPARSE_PAGE_SIZE);
+    if (sparse->size) {
+        result->data = a ? flecs_calloc(a, sparse->size * FLECS_SPARSE_PAGE_SIZE)
+                        : ecs_os_calloc(sparse->size * FLECS_SPARSE_PAGE_SIZE);
+        ecs_assert(result->data != NULL, ECS_INTERNAL_ERROR, NULL);
+    } else {
+        result->data = NULL;
+    }
 
     ecs_assert(result->sparse != NULL, ECS_INTERNAL_ERROR, NULL);
-    ecs_assert(result->data != NULL, ECS_INTERNAL_ERROR, NULL);
 
     return result;
 }
@@ -75,6 +79,9 @@ void flecs_sparse_page_free(
     } else {
         ecs_os_free(page->data);
     }
+
+    page->sparse = NULL;
+    page->data = NULL;
 }
 
 static
@@ -96,7 +103,8 @@ ecs_sparse_page_t* flecs_sparse_get_or_create_page(
 {
     ecs_sparse_page_t *page = flecs_sparse_get_page(sparse, page_index);
     if (page && page->sparse) {
-        ecs_assert(page->data != NULL, ECS_INTERNAL_ERROR, NULL);
+        ecs_assert(!sparse->size || page->data != NULL, 
+            ECS_INTERNAL_ERROR, NULL);
         return page;
     }
 
@@ -110,39 +118,17 @@ void flecs_sparse_grow_dense(
     ecs_vec_append_t(sparse->allocator, &sparse->dense, uint64_t);
 }
 
-static
-uint64_t flecs_sparse_strip_generation(
-    uint64_t *index_out)
-{
-    uint64_t index = *index_out;
-    uint64_t gen = index & ECS_GENERATION_MASK;
-    /* Make sure there's no junk in the id */
-    ecs_assert(gen == (index & (0xFFFFFFFFull << 32)),
-        ECS_INVALID_PARAMETER, NULL);
-    *index_out -= gen;
-    return gen;
-}
-
 static
 void flecs_sparse_assign_index(
     ecs_sparse_page_t * page, 
     uint64_t * dense_array, 
-    uint64_t index, 
+    uint64_t id, 
     int32_t dense)
 {
     /* Initialize sparse-dense pair. This assigns the dense index to the sparse
      * array, and the sparse index to the dense array .*/
-    page->sparse[FLECS_SPARSE_OFFSET(index)] = dense;
-    dense_array[dense] = index;
-}
-
-static
-uint64_t flecs_sparse_inc_gen(
-    uint64_t index)
-{
-    /* When an index is deleted, its generation is increased so that we can do
-     * liveliness checking while recycling ids */
-    return ECS_GENERATION_INC(index);
+    page->sparse[FLECS_SPARSE_OFFSET(id)] = dense;
+    dense_array[dense] = id;
 }
 
 static
@@ -180,16 +166,18 @@ uint64_t flecs_sparse_create_id(
     ecs_sparse_t *sparse,
     int32_t dense)
 {
-    uint64_t index = flecs_sparse_inc_id(sparse);
+    uint64_t id = flecs_sparse_inc_id(sparse);
     flecs_sparse_grow_dense(sparse);
 
-    ecs_sparse_page_t *page = flecs_sparse_get_or_create_page(sparse, FLECS_SPARSE_PAGE(index));
-    ecs_assert(page->sparse[FLECS_SPARSE_OFFSET(index)] == 0, ECS_INTERNAL_ERROR, NULL);
+    ecs_sparse_page_t *page = flecs_sparse_get_or_create_page(
+        sparse, FLECS_SPARSE_PAGE(id));
+    ecs_assert(page->sparse[FLECS_SPARSE_OFFSET(id)] == 0, 
+        ECS_INTERNAL_ERROR, NULL);
     
     uint64_t *dense_array = ecs_vec_first_t(&sparse->dense, uint64_t);
-    flecs_sparse_assign_index(page, dense_array, index, dense);
+    flecs_sparse_assign_index(page, dense_array, id, dense);
     
-    return index;
+    return id;
 }
 
 /* Create new id */
@@ -216,10 +204,11 @@ static
 void* flecs_sparse_get_sparse(
     const ecs_sparse_t *sparse,
     int32_t dense,
-    uint64_t index)
+    uint64_t id)
 {
-    flecs_sparse_strip_generation(&index);
-    ecs_sparse_page_t *page = flecs_sparse_get_page(sparse, FLECS_SPARSE_PAGE(index));
+    uint64_t index = (uint32_t)id;
+    ecs_sparse_page_t *page = flecs_sparse_get_page(
+        sparse, FLECS_SPARSE_PAGE(index));
     if (!page || !page->sparse) {
         return NULL;
     }
@@ -242,12 +231,13 @@ void flecs_sparse_swap_dense(
     int32_t b)
 {
     uint64_t *dense_array = ecs_vec_first_t(&sparse->dense, uint64_t);
-    uint64_t index_a = dense_array[a];
-    uint64_t index_b = dense_array[b];
+    uint64_t id_a = dense_array[a];
+    uint64_t id_b = dense_array[b];
 
-    ecs_sparse_page_t *page_b = flecs_sparse_get_or_create_page(sparse, FLECS_SPARSE_PAGE(index_b));
-    flecs_sparse_assign_index(page_a, dense_array, index_a, b);
-    flecs_sparse_assign_index(page_b, dense_array, index_b, a);
+    ecs_sparse_page_t *page_b = flecs_sparse_get_or_create_page(
+            sparse, FLECS_SPARSE_PAGE(id_b));
+    flecs_sparse_assign_index(page_a, dense_array, id_a, b);
+    flecs_sparse_assign_index(page_b, dense_array, id_b, a);
 }
 
 void flecs_sparse_init(
@@ -321,11 +311,11 @@ void* flecs_sparse_add(
 {
     ecs_assert(sparse != NULL, ECS_INVALID_PARAMETER, NULL);
     ecs_assert(!size || size == sparse->size, ECS_INVALID_PARAMETER, NULL);
-    uint64_t index = flecs_sparse_new_index(sparse);
-    ecs_sparse_page_t *page = flecs_sparse_get_page(sparse, FLECS_SPARSE_PAGE(index));
+    uint64_t id = flecs_sparse_new_index(sparse);
+    ecs_sparse_page_t *page = flecs_sparse_get_page(sparse, FLECS_SPARSE_PAGE(id));
     ecs_assert(page != NULL, ECS_INTERNAL_ERROR, NULL);
     ecs_assert(page->data != NULL, ECS_INTERNAL_ERROR, NULL);
-    return DATA(page->data, size, FLECS_SPARSE_OFFSET(index));
+    return DATA(page->data, size, FLECS_SPARSE_OFFSET(id));
 }
 
 uint64_t flecs_sparse_last_id(
@@ -336,18 +326,33 @@ uint64_t flecs_sparse_last_id(
     return dense_array[sparse->count - 1];
 }
 
+void* flecs_sparse_insert(
+    ecs_sparse_t *sparse,
+    ecs_size_t size,
+    uint64_t id)
+{
+    bool is_new = true;
+    void *result = flecs_sparse_ensure(sparse, size, id, &is_new);
+    if (!is_new) {
+        result = NULL;
+    }
+    return result;
+}
+
 void* flecs_sparse_ensure(
     ecs_sparse_t *sparse,
     ecs_size_t size,
-    uint64_t index)
+    uint64_t id,
+    bool *is_new)
 {
     ecs_assert(sparse != NULL, ECS_INVALID_PARAMETER, NULL);
     ecs_assert(!size || size == sparse->size, ECS_INVALID_PARAMETER, NULL);
     ecs_assert(ecs_vec_count(&sparse->dense) > 0, ECS_INTERNAL_ERROR, NULL);
     (void)size;
 
-    uint64_t gen = flecs_sparse_strip_generation(&index);
-    ecs_sparse_page_t *page = flecs_sparse_get_or_create_page(sparse, FLECS_SPARSE_PAGE(index));
+    uint64_t index = (uint32_t)id;
+    ecs_sparse_page_t *page = flecs_sparse_get_or_create_page(
+        sparse, FLECS_SPARSE_PAGE(index));
     int32_t offset = FLECS_SPARSE_OFFSET(index);
     int32_t dense = page->sparse[offset];
 
@@ -364,19 +369,10 @@ void* flecs_sparse_ensure(
             sparse->count ++;
 
             /* Set dense element to new generation */
-            ecs_vec_first_t(&sparse->dense, uint64_t)[dense] = index | gen;
+            ecs_vec_first_t(&sparse->dense, uint64_t)[dense] = id;
         } else {
-            /* Dense is already alive, nothing to be done */
+            if (is_new) *is_new = false;
         }
-
-        /* Ensure provided generation matches current. Only allow mismatching
-         * generations if the provided generation count is 0. This allows for
-         * using the ensure function in combination with ids that have their
-         * generation stripped. */
-#ifdef FLECS_DEBUG
-        uint64_t *dense_array = ecs_vec_first_t(&sparse->dense, uint64_t);
-        ecs_assert(!gen || dense_array[dense] == (index | gen), ECS_INTERNAL_ERROR, NULL);
-#endif
     } else {
         /* Element is not paired yet. Must add a new element to dense array */
         flecs_sparse_grow_dense(sparse);
@@ -394,12 +390,13 @@ void* flecs_sparse_ensure(
             /* If there are unused elements in the list, move the first unused
              * element to the end of the list */
             uint64_t unused = dense_array[count];
-            ecs_sparse_page_t *unused_page = flecs_sparse_get_or_create_page(sparse, FLECS_SPARSE_PAGE(unused));
-            flecs_sparse_assign_index(unused_page, dense_array, unused, dense_count);
+            ecs_sparse_page_t *unused_page = flecs_sparse_get_or_create_page(
+                sparse, FLECS_SPARSE_PAGE(unused));
+            flecs_sparse_assign_index(
+                unused_page, dense_array, unused, dense_count);
         }
 
         flecs_sparse_assign_index(page, dense_array, index, count);
-        dense_array[count] |= gen;
     }
 
     return DATA(page->data, sparse->size, offset);
@@ -408,15 +405,16 @@ void* flecs_sparse_ensure(
 void* flecs_sparse_ensure_fast(
     ecs_sparse_t *sparse,
     ecs_size_t size,
-    uint64_t index_long)
+    uint64_t id)
 {
     ecs_assert(sparse != NULL, ECS_INVALID_PARAMETER, NULL);
     ecs_assert(!size || size == sparse->size, ECS_INVALID_PARAMETER, NULL);
     ecs_assert(ecs_vec_count(&sparse->dense) > 0, ECS_INTERNAL_ERROR, NULL);
     (void)size;
 
-    uint32_t index = (uint32_t)index_long;
-    ecs_sparse_page_t *page = flecs_sparse_get_or_create_page(sparse, FLECS_SPARSE_PAGE(index));
+    uint32_t index = (uint32_t)id;
+    ecs_sparse_page_t *page = flecs_sparse_get_or_create_page(
+        sparse, FLECS_SPARSE_PAGE(index));
     int32_t offset = FLECS_SPARSE_OFFSET(index);
     int32_t dense = page->sparse[offset];
     int32_t count = sparse->count;
@@ -435,21 +433,22 @@ void* flecs_sparse_ensure_fast(
     return DATA(page->data, sparse->size, offset);
 }
 
-void* flecs_sparse_remove_fast(
+bool flecs_sparse_remove(
     ecs_sparse_t *sparse,
     ecs_size_t size,
-    uint64_t index)
+    uint64_t id)
 {
     ecs_assert(sparse != NULL, ECS_INVALID_PARAMETER, NULL);
     ecs_assert(!size || size == sparse->size, ECS_INVALID_PARAMETER, NULL);
     (void)size;
 
-    ecs_sparse_page_t *page = flecs_sparse_get_page(sparse, FLECS_SPARSE_PAGE(index));
+    ecs_sparse_page_t *page = flecs_sparse_get_page(
+        sparse, FLECS_SPARSE_PAGE(id));
     if (!page || !page->sparse) {
-        return NULL;
+        return false;
     }
 
-    flecs_sparse_strip_generation(&index);
+    uint64_t index = (uint32_t)id;
     int32_t offset = FLECS_SPARSE_OFFSET(index);
     int32_t dense = page->sparse[offset];
 
@@ -465,46 +464,54 @@ void* flecs_sparse_remove_fast(
         }
 
         /* Reset memory to zero on remove */
-        return DATA(page->data, sparse->size, offset);
+        if (sparse->size) {
+            void *ptr = DATA(page->data, sparse->size, offset);
+            ecs_os_memset(ptr, 0, size);
+        }
+
+        /* Reset memory to zero on remove */
+        return true;
     } else {
         /* Element is not paired and thus not alive, nothing to be done */
-        return NULL;
+        return false;
     }
 }
 
+static
+uint64_t flecs_sparse_inc_gen(
+    uint64_t index)
+{
+    /* When an index is deleted, its generation is increased so that we can do
+     * liveliness checking while recycling ids */
+    return ECS_GENERATION_INC(index);
+}
 
-void flecs_sparse_remove(
+bool flecs_sparse_remove_w_gen(
     ecs_sparse_t *sparse,
     ecs_size_t size,
-    uint64_t index)
+    uint64_t id)
 {
     ecs_assert(sparse != NULL, ECS_INVALID_PARAMETER, NULL);
     ecs_assert(!size || size == sparse->size, ECS_INVALID_PARAMETER, NULL);
     (void)size;
 
-    ecs_sparse_page_t *page = flecs_sparse_get_page(sparse, FLECS_SPARSE_PAGE(index));
+    ecs_sparse_page_t *page = flecs_sparse_get_page(
+        sparse, FLECS_SPARSE_PAGE(id));
     if (!page || !page->sparse) {
-        return;
+        return false;
     }
 
-    uint64_t gen = flecs_sparse_strip_generation(&index);
+    uint64_t index = (uint32_t)id;
     int32_t offset = FLECS_SPARSE_OFFSET(index);
     int32_t dense = page->sparse[offset];
 
     if (dense) {
-        uint64_t *dense_array = ecs_vec_first_t(&sparse->dense, uint64_t);
-        uint64_t cur_gen = dense_array[dense] & ECS_GENERATION_MASK;
-        if (gen != cur_gen) {
-            /* Generation doesn't match which means that the provided entity is
-             * already not alive. */
-            return;
-        }
-
         /* Increase generation */
-        dense_array[dense] = index | flecs_sparse_inc_gen(cur_gen);
-        
+        uint64_t *dense_array = ecs_vec_first_t(&sparse->dense, uint64_t);
+        ecs_assert(dense_array[dense] == id, ECS_INVALID_PARAMETER, NULL);
+        dense_array[dense] = flecs_sparse_inc_gen(id);
+
         int32_t count = sparse->count;
-        
         if (dense == (count - 1)) {
             /* If dense is the last used element, simply decrease count */
             sparse->count --;
@@ -513,16 +520,19 @@ void flecs_sparse_remove(
             flecs_sparse_swap_dense(sparse, page, dense, count - 1);
             sparse->count --;
         } else {
-            /* Element is not alive, nothing to be done */
-            return;
+            return false;
         }
 
         /* Reset memory to zero on remove */
-        void *ptr = DATA(page->data, sparse->size, offset);
-        ecs_os_memset(ptr, 0, size);
+        if (sparse->size) {
+            void *ptr = DATA(page->data, sparse->size, offset);
+            ecs_os_memset(ptr, 0, size);
+        }
+
+        return true;
     } else {
         /* Element is not paired and thus not alive, nothing to be done */
-        return;
+        return false;
     }
 }
 
@@ -539,102 +549,42 @@ void* flecs_sparse_get_dense(
     dense_index ++;
 
     uint64_t *dense_array = ecs_vec_first_t(&sparse->dense, uint64_t);
-    return flecs_sparse_get_sparse(sparse, dense_index, dense_array[dense_index]);
+    return flecs_sparse_get_sparse(
+        sparse, dense_index, dense_array[dense_index]);
 }
 
 bool flecs_sparse_is_alive(
     const ecs_sparse_t *sparse,
-    uint64_t index)
+    uint64_t id)
 {
-    ecs_sparse_page_t *page = flecs_sparse_get_page(sparse, FLECS_SPARSE_PAGE(index));
+    ecs_sparse_page_t *page = flecs_sparse_get_page(
+        sparse, FLECS_SPARSE_PAGE(id));
     if (!page || !page->sparse) {
         return false;
     }
 
-    int32_t offset = FLECS_SPARSE_OFFSET(index);
+    int32_t offset = FLECS_SPARSE_OFFSET(id);
     int32_t dense = page->sparse[offset];
     if (!dense || (dense >= sparse->count)) {
         return false;
     }
 
-    uint64_t gen = flecs_sparse_strip_generation(&index);
-    uint64_t *dense_array = ecs_vec_first_t(&sparse->dense, uint64_t);
-    uint64_t cur_gen = dense_array[dense] & ECS_GENERATION_MASK;
-
-    if (cur_gen != gen) {
-        return false;
-    }
-
     ecs_assert(dense == page->sparse[offset], ECS_INTERNAL_ERROR, NULL);
     return true;
 }
 
-void* flecs_sparse_try(
-    const ecs_sparse_t *sparse,
-    ecs_size_t size,
-    uint64_t index)
-{
-    ecs_assert(sparse != NULL, ECS_INVALID_PARAMETER, NULL);
-    ecs_assert(!size || size == sparse->size, ECS_INVALID_PARAMETER, NULL);
-    (void)size;
-    ecs_sparse_page_t *page = flecs_sparse_get_page(sparse, FLECS_SPARSE_PAGE(index));
-    if (!page || !page->sparse) {
-        return NULL;
-    }
-
-    int32_t offset = FLECS_SPARSE_OFFSET(index);
-    int32_t dense = page->sparse[offset];
-    if (!dense || (dense >= sparse->count)) {
-        return NULL;
-    }
-
-    uint64_t gen = flecs_sparse_strip_generation(&index);
-    uint64_t *dense_array = ecs_vec_first_t(&sparse->dense, uint64_t);
-    uint64_t cur_gen = dense_array[dense] & ECS_GENERATION_MASK;
-    if (cur_gen != gen) {
-        return NULL;
-    }
-
-    ecs_assert(dense == page->sparse[offset], ECS_INTERNAL_ERROR, NULL);
-    return DATA(page->data, sparse->size, offset);
-}
-
 void* flecs_sparse_get(
     const ecs_sparse_t *sparse,
     ecs_size_t size,
-    uint64_t index)
-{
-    ecs_assert(sparse != NULL, ECS_INVALID_PARAMETER, NULL);
-    ecs_assert(!size || size == sparse->size, ECS_INVALID_PARAMETER, NULL);
-    (void)size;
-
-    ecs_sparse_page_t *page = ecs_vec_get_t(&sparse->pages, ecs_sparse_page_t, FLECS_SPARSE_PAGE(index));
-    int32_t offset = FLECS_SPARSE_OFFSET(index);
-    int32_t dense = page->sparse[offset];
-    ecs_assert(dense != 0, ECS_INTERNAL_ERROR, NULL);
-
-    uint64_t gen = flecs_sparse_strip_generation(&index);
-    uint64_t *dense_array = ecs_vec_first_t(&sparse->dense, uint64_t);
-    uint64_t cur_gen = dense_array[dense] & ECS_GENERATION_MASK;
-    (void)cur_gen; (void)gen;
-
-    ecs_assert(cur_gen == gen, ECS_INVALID_PARAMETER, NULL);
-    ecs_assert(dense == page->sparse[offset], ECS_INTERNAL_ERROR, NULL);
-    ecs_assert(dense < sparse->count, ECS_INTERNAL_ERROR, NULL);
-    return DATA(page->data, sparse->size, offset);
-}
-
-void* flecs_sparse_get_any(
-    const ecs_sparse_t *sparse,
-    ecs_size_t size,
-    uint64_t index)
+    uint64_t id)
 {
     ecs_assert(sparse != NULL, ECS_INVALID_PARAMETER, NULL);
     ecs_assert(!size || size == sparse->size, ECS_INVALID_PARAMETER, NULL);
     (void)size;
     
-    flecs_sparse_strip_generation(&index);
-    ecs_sparse_page_t *page = flecs_sparse_get_page(sparse, FLECS_SPARSE_PAGE(index));
+    uint64_t index = (uint32_t)id;
+    ecs_sparse_page_t *page = flecs_sparse_get_page(
+        sparse, FLECS_SPARSE_PAGE(index));
     if (!page || !page->sparse) {
         return NULL;
     }
@@ -650,6 +600,24 @@ void* flecs_sparse_get_any(
     return DATA(page->data, sparse->size, offset);
 }
 
+bool flecs_sparse_has(
+    const ecs_sparse_t *sparse,
+    uint64_t id)
+{
+    ecs_assert(sparse != NULL, ECS_INVALID_PARAMETER, NULL);
+    
+    uint64_t index = (uint32_t)id;
+    ecs_sparse_page_t *page = flecs_sparse_get_page(sparse, 
+        FLECS_SPARSE_PAGE(index));
+    if (!page || !page->sparse) {
+        return false;
+    }
+
+    int32_t offset = FLECS_SPARSE_OFFSET(id);
+    int32_t dense = page->sparse[offset];
+    return dense && (dense < sparse->count);
+}
+
 int32_t flecs_sparse_count(
     const ecs_sparse_t *sparse)
 {
@@ -674,7 +642,7 @@ const uint64_t* flecs_sparse_ids(
 void flecs_sparse_shrink(
     ecs_sparse_t *sparse)
 {
-    int32_t i, e, max_page_index = 0, count = ecs_vec_count(&sparse->pages);
+    int32_t i, e, max_page_index = -1, count = ecs_vec_count(&sparse->pages);
     ecs_sparse_page_t *pages = ecs_vec_first_t(&sparse->pages,
         ecs_sparse_page_t);
     for (i = 0; i < count; i ++) {
@@ -686,10 +654,10 @@ void flecs_sparse_shrink(
 
         bool has_alive = false;
         for (e = 0; e < FLECS_SPARSE_PAGE_SIZE; e ++) {
-            uint64_t index = 
+            uint64_t id = 
                 flecs_ito(uint64_t, (i * FLECS_SPARSE_PAGE_SIZE) + e);
 
-            if (flecs_sparse_is_alive(sparse, index)) {
+            if (flecs_sparse_is_alive(sparse, id)) {
                 has_alive = true;
                 break;
             }
