@@ -58,7 +58,7 @@ ecs_event_record_t* flecs_event_record_get(
     else if (event == EcsWildcard) return ECS_CONST_CAST(ecs_event_record_t*, &o->on_wildcard);
 
     /* User events */
-    return flecs_sparse_try_t(&o->events, ecs_event_record_t, event);
+    return flecs_sparse_get_t(&o->events, ecs_event_record_t, event);
 }
 
 ecs_event_record_t* flecs_event_record_ensure(
@@ -71,7 +71,10 @@ ecs_event_record_t* flecs_event_record_ensure(
     if (er) {
         return er;
     }
-    er = flecs_sparse_ensure_t(&o->events, ecs_event_record_t, event);
+    er = flecs_sparse_get_t(&o->events, ecs_event_record_t, event);
+    if (!er) {
+        er = flecs_sparse_ensure_t(&o->events, ecs_event_record_t, event, NULL);
+    }
     er->event = event;
     return er;
 }
@@ -245,8 +248,8 @@ static
 void flecs_emit_propagate(
     ecs_world_t *world,
     ecs_iter_t *it,
-    ecs_component_record_t *cdr,
-    ecs_component_record_t *tgt_idr,
+    ecs_component_record_t *cr,
+    ecs_component_record_t *tgt_cr,
     ecs_entity_t trav,
     ecs_event_id_record_t **iders,
     int32_t ider_count);
@@ -255,7 +258,7 @@ static
 void flecs_emit_propagate_id(
     ecs_world_t *world,
     ecs_iter_t *it,
-    ecs_component_record_t *cdr,
+    ecs_component_record_t *cr,
     ecs_component_record_t *cur,
     ecs_entity_t trav,
     ecs_event_id_record_t **iders,
@@ -274,7 +277,7 @@ void flecs_emit_propagate_id(
             continue;
         }
 
-        bool owned = flecs_component_get_table(cdr, table) != NULL;
+        bool owned = flecs_component_get_table(cr, table) != NULL;
 
         int32_t e, entity_count = ecs_table_count(table);
         it->table = table;
@@ -305,11 +308,11 @@ void flecs_emit_propagate_id(
         for (e = 0; e < entity_count; e ++) {
             ecs_record_t *r = flecs_entities_get(world, entities[e]);
             ecs_assert(r != NULL, ECS_INTERNAL_ERROR, NULL);
-            ecs_component_record_t *idr_t = r->cdr;
-            if (idr_t) {
+            ecs_component_record_t *cr_t = r->cr;
+            if (cr_t) {
                 /* Only notify for entities that are used in pairs with
                  * traversable relationships */
-                flecs_emit_propagate(world, it, cdr, idr_t, trav,
+                flecs_emit_propagate(world, it, cr, cr_t, trav,
                     iders, ider_count);
             }
         }
@@ -323,16 +326,16 @@ static
 void flecs_emit_propagate(
     ecs_world_t *world,
     ecs_iter_t *it,
-    ecs_component_record_t *cdr,
-    ecs_component_record_t *tgt_idr,
+    ecs_component_record_t *cr,
+    ecs_component_record_t *tgt_cr,
     ecs_entity_t propagate_trav,
     ecs_event_id_record_t **iders,
     int32_t ider_count)
 {
-    ecs_assert(tgt_idr != NULL, ECS_INTERNAL_ERROR, NULL);    
+    ecs_assert(tgt_cr != NULL, ECS_INTERNAL_ERROR, NULL);    
 
     if (ecs_should_log_3()) {
-        char *idstr = ecs_id_str(world, tgt_idr->id);
+        char *idstr = ecs_id_str(world, tgt_cr->id);
         ecs_dbg_3("propagate events/invalidate cache for %s", idstr);
         ecs_os_free(idstr);
     }
@@ -340,7 +343,7 @@ void flecs_emit_propagate(
     ecs_log_push_3();
 
     /* Propagate to records of traversable relationships */
-    ecs_component_record_t *cur = tgt_idr;
+    ecs_component_record_t *cur = tgt_cr;
     while ((cur = flecs_component_trav_next(cur))) {
         cur->pair->reachable.generation ++; /* Invalidate cache */
 
@@ -353,7 +356,7 @@ void flecs_emit_propagate(
         }
 
         flecs_emit_propagate_id(
-            world, it, cdr, cur, trav, iders, ider_count);
+            world, it, cr, cur, trav, iders, ider_count);
     }
 
     ecs_log_pop_3();
@@ -362,18 +365,18 @@ void flecs_emit_propagate(
 static
 void flecs_emit_propagate_invalidate_tables(
     ecs_world_t *world,
-    ecs_component_record_t *tgt_idr)
+    ecs_component_record_t *tgt_cr)
 {
-    ecs_assert(tgt_idr != NULL, ECS_INTERNAL_ERROR, NULL);
+    ecs_assert(tgt_cr != NULL, ECS_INTERNAL_ERROR, NULL);
 
     if (ecs_should_log_3()) {
-        char *idstr = ecs_id_str(world, tgt_idr->id);
+        char *idstr = ecs_id_str(world, tgt_cr->id);
         ecs_dbg_3("invalidate reachable cache for %s", idstr);
         ecs_os_free(idstr);
     }
 
     /* Invalidate records of traversable relationships */
-    ecs_component_record_t *cur = tgt_idr;
+    ecs_component_record_t *cur = tgt_cr;
     while ((cur = flecs_component_trav_next(cur))) {
         ecs_reachable_cache_t *rc = &cur->pair->reachable;
         if (rc->current != rc->generation) {
@@ -400,11 +403,11 @@ void flecs_emit_propagate_invalidate_tables(
 
             for (e = 0; e < entity_count; e ++) {
                 ecs_record_t *r = flecs_entities_get(world, entities[e]);
-                ecs_component_record_t *idr_t = r->cdr;
-                if (idr_t) {
+                ecs_component_record_t *cr_t = r->cr;
+                if (cr_t) {
                     /* Only notify for entities that are used in pairs with
                      * traversable relationships */
-                    flecs_emit_propagate_invalidate_tables(world, idr_t);
+                    flecs_emit_propagate_invalidate_tables(world, cr_t);
                 }
             }
         }
@@ -427,10 +430,10 @@ void flecs_emit_propagate_invalidate(
             continue;
         }
 
-        ecs_component_record_t *idr_t = record->cdr;
-        if (idr_t) {
+        ecs_component_record_t *cr_t = record->cr;
+        if (cr_t) {
             /* Event is used as target in traversable relationship, propagate */
-            flecs_emit_propagate_invalidate_tables(world, idr_t);
+            flecs_emit_propagate_invalidate_tables(world, cr_t);
         }
     }
 }
@@ -439,7 +442,7 @@ static
 void flecs_propagate_entities(
     ecs_world_t *world,
     ecs_iter_t *it,
-    ecs_component_record_t *cdr,
+    ecs_component_record_t *cr,
     const ecs_entity_t *entities,
     int32_t count,
     ecs_entity_t src,
@@ -466,13 +469,13 @@ void flecs_propagate_entities(
             continue;
         }
 
-        ecs_component_record_t *idr_t = record->cdr;
-        if (idr_t) {
+        ecs_component_record_t *cr_t = record->cr;
+        if (cr_t) {
             /* Entity is used as target in traversable pairs, propagate */
             ecs_entity_t e = src ? src : entities[i];
             it->sources[0] = e;
             flecs_emit_propagate(
-                world, it, cdr, idr_t, 0, iders, ider_count);
+                world, it, cr, cr_t, 0, iders, ider_count);
         }
     }
     
@@ -515,8 +518,8 @@ void flecs_override_copy(
     ecs_iter_action_t on_set = ti->hooks.on_set;
     if (on_set) {
         const ecs_entity_t *entities = &ecs_table_entities(table)[offset];
-        flecs_invoke_hook(world, table, tr, count, offset, entities,
-            ti->component, ti, EcsOnSet, on_set);
+        flecs_invoke_hook(world, table, (ecs_component_record_t*)tr->hdr.cache, tr, 
+            count, offset, entities, ti->component, ti, EcsOnSet, on_set);
     }
 }
 
@@ -526,7 +529,7 @@ void* flecs_override(
     const ecs_type_t *emit_ids,
     ecs_id_t id,
     ecs_table_t *table,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
     if (it->event != EcsOnAdd || (it->flags & EcsEventNoOnSet)) {
         return NULL;
@@ -545,7 +548,7 @@ void* flecs_override(
              * after it was inherited, as this does not change the actual value
              * of the component for the entity (it is copied from the existing
              * overridden component), and does not require an OnSet event. */
-            const ecs_table_record_t *tr = flecs_component_get_table(cdr, table);
+            const ecs_table_record_t *tr = flecs_component_get_table(cr, table);
             if (!tr) {
                 continue;
             }
@@ -570,7 +573,7 @@ void flecs_emit_forward_up(
     const ecs_type_t *emit_ids,
     ecs_iter_t *it,
     ecs_table_t *table,
-    ecs_component_record_t *cdr,
+    ecs_component_record_t *cr,
     ecs_vec_t *stack,
     ecs_vec_t *reachable_ids,
     int32_t depth);
@@ -583,13 +586,13 @@ void flecs_emit_forward_id(
     const ecs_type_t *emit_ids,
     ecs_iter_t *it,
     ecs_table_t *table,
-    ecs_component_record_t *cdr,
+    ecs_component_record_t *cr,
     ecs_entity_t tgt,
     ecs_table_t *tgt_table,
     int32_t column,
     ecs_entity_t trav)
 {
-    ecs_id_t id = cdr->id;
+    ecs_id_t id = cr->id;
     ecs_entity_t event = er ? er->event : 0;
     bool inherit = trav == EcsIsA;
     bool may_override = inherit && (event == EcsOnAdd) && (emit_ids->count > 1);
@@ -618,13 +621,13 @@ void flecs_emit_forward_id(
 
     int32_t storage_i = ecs_table_type_to_column_index(tgt_table, column);
     if (storage_i != -1) {
-        ecs_assert(cdr->type_info != NULL, ECS_INTERNAL_ERROR, NULL);
+        ecs_assert(cr->type_info != NULL, ECS_INTERNAL_ERROR, NULL);
         ecs_column_t *c = &tgt_table->data.columns[storage_i];
         it->trs[0] = &tgt_table->_->records[column];
         ECS_CONST_CAST(int32_t*, it->sizes)[0] = c->ti->size; /* safe, see above */
     }
 
-    const ecs_table_record_t *tr = flecs_component_get_table(cdr, table);
+    const ecs_table_record_t *tr = flecs_component_get_table(cr, table);
     bool owned = tr != NULL;
 
     for (ider_i = 0; ider_i < ider_count; ider_i ++) {
@@ -644,7 +647,7 @@ void flecs_emit_forward_id(
         /* If component was added together with IsA relationship, still emit
          * OnSet event, as it's a new value for the entity. */
         const ecs_table_record_t *base_tr = it->trs[0];
-        void *ptr = flecs_override(it, emit_ids, id, table, cdr);
+        void *ptr = flecs_override(it, emit_ids, id, table, cr);
         if (ptr) {
             override = true;
         }
@@ -688,7 +691,7 @@ void flecs_emit_forward_and_cache_id(
     const ecs_type_t *emit_ids,
     ecs_iter_t *it,
     ecs_table_t *table,
-    ecs_component_record_t *cdr,
+    ecs_component_record_t *cr,
     ecs_entity_t tgt,
     ecs_record_t *tgt_record,
     ecs_table_t *tgt_table,
@@ -703,27 +706,27 @@ void flecs_emit_forward_and_cache_id(
     elem->tr = tgt_tr;
     elem->record = tgt_record;
     elem->src = tgt;
-    elem->id = cdr->id;
+    elem->id = cr->id;
 #ifndef NDEBUG
     elem->table = tgt_table;
 #endif
     ecs_assert(tgt_table == tgt_record->table, ECS_INTERNAL_ERROR, NULL);
 
-    flecs_emit_forward_id(world, er, er_onset, emit_ids, it, table, cdr,
+    flecs_emit_forward_id(world, er, er_onset, emit_ids, it, table, cr,
         tgt, tgt_table, column, trav);
 }
 
 static
 int32_t flecs_emit_stack_at(
     ecs_vec_t *stack,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
     int32_t sp = 0, stack_count = ecs_vec_count(stack);
     ecs_table_t **stack_elems = ecs_vec_first(stack);
 
     for (sp = 0; sp < stack_count; sp ++) {
         ecs_table_t *elem = stack_elems[sp];
-        if (flecs_component_get_table(cdr, elem)) {
+        if (flecs_component_get_table(cr, elem)) {
             break;
         }
     }
@@ -734,9 +737,9 @@ int32_t flecs_emit_stack_at(
 static
 bool flecs_emit_stack_has(
     ecs_vec_t *stack,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
-    return flecs_emit_stack_at(stack, cdr) != ecs_vec_count(stack);
+    return flecs_emit_stack_at(stack, cr) != ecs_vec_count(stack);
 }
 
 static
@@ -759,22 +762,22 @@ void flecs_emit_forward_cached_ids(
         ecs_reachable_elem_t *rc_elem = &elems[i];
         const ecs_table_record_t *rc_tr = rc_elem->tr;
         ecs_assert(rc_tr != NULL, ECS_INTERNAL_ERROR, NULL);
-        ecs_component_record_t *rc_idr = (ecs_component_record_t*)rc_tr->hdr.cache;
+        ecs_component_record_t *rc_cr = (ecs_component_record_t*)rc_tr->hdr.cache;
         ecs_record_t *rc_record = rc_elem->record;
 
-        ecs_assert(rc_idr->id == rc_elem->id, ECS_INTERNAL_ERROR, NULL);
+        ecs_assert(rc_cr->id == rc_elem->id, ECS_INTERNAL_ERROR, NULL);
         ecs_assert(rc_record != NULL, ECS_INTERNAL_ERROR, NULL);
         ecs_assert(flecs_entities_get(world, rc_elem->src) == 
             rc_record, ECS_INTERNAL_ERROR, NULL);
         ecs_dbg_assert(rc_record->table == rc_elem->table, 
             ECS_INTERNAL_ERROR, NULL);
 
-        if (flecs_emit_stack_has(stack, rc_idr)) {
+        if (flecs_emit_stack_has(stack, rc_cr)) {
             continue;
         }
 
         flecs_emit_forward_and_cache_id(world, er, er_onset, emit_ids,
-            it, table, rc_idr, rc_elem->src,
+            it, table, rc_cr, rc_elem->src,
                 rc_record, rc_record->table, rc_tr, rc_tr->index,
                     reachable_ids, trav);
     }
@@ -819,7 +822,7 @@ void flecs_emit_forward_table_up(
     ecs_entity_t tgt,
     ecs_table_t *tgt_table,
     ecs_record_t *tgt_record,
-    ecs_component_record_t *tgt_idr,
+    ecs_component_record_t *tgt_cr,
     ecs_vec_t *stack,
     ecs_vec_t *reachable_ids,
     int32_t depth)
@@ -830,10 +833,10 @@ void flecs_emit_forward_table_up(
     int32_t rc_child_offset = ecs_vec_count(reachable_ids);
     int32_t stack_count = ecs_vec_count(stack);
 
-    /* If tgt_idr is out of sync but is not the current component record being updated,
+    /* If tgt_cr is out of sync but is not the current component record being updated,
      * keep track so that we can update two records for the cost of one. */
-    ecs_assert(tgt_idr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
-    ecs_reachable_cache_t *rc = &tgt_idr->pair->reachable;
+    ecs_assert(tgt_cr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
+    ecs_reachable_cache_t *rc = &tgt_cr->pair->reachable;
     bool parent_revalidate = (reachable_ids != &rc->ids) && 
         (rc->current != rc->generation);
     if (parent_revalidate) {
@@ -841,7 +844,7 @@ void flecs_emit_forward_table_up(
     }
 
     if (ecs_should_log_3()) {
-        char *idstr = ecs_id_str(world, tgt_idr->id);
+        char *idstr = ecs_id_str(world, tgt_cr->id);
         ecs_dbg_3("forward events from %s", idstr);
         ecs_os_free(idstr);
     }
@@ -849,20 +852,20 @@ void flecs_emit_forward_table_up(
 
     /* Function may have to copy values from overridden components if an IsA
      * relationship was added together with other components. */
-    ecs_entity_t trav = ECS_PAIR_FIRST(tgt_idr->id);
+    ecs_entity_t trav = ECS_PAIR_FIRST(tgt_cr->id);
     bool inherit = trav == EcsIsA;
 
     for (i = 0; i < id_count; i ++) {
         ecs_id_t id = ids[i];
         ecs_table_record_t *tgt_tr = &tgt_table->_->records[i];
-        ecs_component_record_t *cdr = (ecs_component_record_t*)tgt_tr->hdr.cache;
-        if (inherit && !(cdr->flags & EcsIdOnInstantiateInherit)) {
+        ecs_component_record_t *cr = (ecs_component_record_t*)tgt_tr->hdr.cache;
+        if (inherit && !(cr->flags & EcsIdOnInstantiateInherit)) {
             continue;
         }
 
-        if (cdr == tgt_idr) {
-            char *idstr = ecs_id_str(world, cdr->id);
-            ecs_assert(cdr != tgt_idr, ECS_CYCLE_DETECTED, idstr);
+        if (cr == tgt_cr) {
+            char *idstr = ecs_id_str(world, cr->id);
+            ecs_assert(cr != tgt_cr, ECS_CYCLE_DETECTED, idstr);
             ecs_os_free(idstr);
             return;
         }
@@ -873,9 +876,9 @@ void flecs_emit_forward_table_up(
                 ecs_table_t*);
             t[0] = tgt_table;
 
-            ecs_assert(cdr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
-            ecs_reachable_cache_t *idr_rc = &cdr->pair->reachable;
-            if (idr_rc->current == idr_rc->generation) {
+            ecs_assert(cr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
+            ecs_reachable_cache_t *cr_rc = &cr->pair->reachable;
+            if (cr_rc->current == cr_rc->generation) {
                 /* Cache hit, use cached ids to prevent traversing the same
                  * hierarchy multiple times. This especially speeds up code 
                  * where (deep) hierarchies are created. */
@@ -886,13 +889,13 @@ void flecs_emit_forward_table_up(
                 }
                 ecs_log_push_3();
                 flecs_emit_forward_cached_ids(world, er, er_onset, emit_ids, it,
-                    table, idr_rc, reachable_ids, stack, trav);
+                    table, cr_rc, reachable_ids, stack, trav);
                 ecs_log_pop_3();
             } else {
                 /* Cache is dirty, traverse upwards */
                 do {
                     flecs_emit_forward_up(world, er, er_onset, emit_ids, it, 
-                        table, cdr, stack, reachable_ids, depth);
+                        table, cr, stack, reachable_ids, depth);
                     if (++i >= id_count) {
                         break;
                     }
@@ -908,7 +911,7 @@ void flecs_emit_forward_table_up(
             continue;
         }
 
-        int32_t stack_at = flecs_emit_stack_at(stack, cdr);
+        int32_t stack_at = flecs_emit_stack_at(stack, cr);
         if (parent_revalidate && (stack_at == (stack_count - 1))) {
             /* If parent component record needs to be revalidated, add id */
             ecs_reachable_elem_t *elem = ecs_vec_append_t(a, &rc->ids, 
@@ -916,7 +919,7 @@ void flecs_emit_forward_table_up(
             elem->tr = tgt_tr;
             elem->record = tgt_record;
             elem->src = tgt;
-            elem->id = cdr->id;
+            elem->id = cr->id;
 #ifndef NDEBUG
             elem->table = tgt_table;
 #endif
@@ -928,7 +931,7 @@ void flecs_emit_forward_table_up(
         }
 
         flecs_emit_forward_and_cache_id(world, er, er_onset, emit_ids, it,
-            table, cdr, tgt, tgt_record, tgt_table, tgt_tr, i, 
+            table, cr, tgt, tgt_record, tgt_table, tgt_tr, i, 
                 reachable_ids, trav);
     }
 
@@ -954,7 +957,7 @@ void flecs_emit_forward_table_up(
         rc->current = rc->generation;
 
         if (ecs_should_log_3()) {
-            char *idstr = ecs_id_str(world, tgt_idr->id);
+            char *idstr = ecs_id_str(world, tgt_cr->id);
             ecs_dbg_3("cache revalidated for %s:", idstr);
             ecs_os_free(idstr);
             flecs_emit_dump_cache(world, &rc->ids);
@@ -972,19 +975,19 @@ void flecs_emit_forward_up(
     const ecs_type_t *emit_ids,
     ecs_iter_t *it,
     ecs_table_t *table,
-    ecs_component_record_t *cdr,
+    ecs_component_record_t *cr,
     ecs_vec_t *stack,
     ecs_vec_t *reachable_ids,
     int32_t depth)
 {
     if (depth >= FLECS_DAG_DEPTH_MAX) {
-        char *idstr = ecs_id_str(world, cdr->id);
+        char *idstr = ecs_id_str(world, cr->id);
         ecs_assert(depth < FLECS_DAG_DEPTH_MAX, ECS_CYCLE_DETECTED, idstr);
         ecs_os_free(idstr);
         return;
     }
 
-    ecs_id_t id = cdr->id;
+    ecs_id_t id = cr->id;
     ecs_entity_t tgt = ECS_PAIR_SECOND(id);
     tgt = flecs_entities_get_alive(world, tgt);
     ecs_assert(tgt != 0, ECS_INTERNAL_ERROR, NULL);
@@ -995,7 +998,7 @@ void flecs_emit_forward_up(
     }
 
     flecs_emit_forward_table_up(world, er, er_onset, emit_ids, it, table, 
-        tgt, tgt_table, tgt_record, cdr, stack, reachable_ids, depth + 1);
+        tgt, tgt_table, tgt_record, cr, stack, reachable_ids, depth + 1);
 }
 
 static
@@ -1006,15 +1009,15 @@ void flecs_emit_forward(
     const ecs_type_t *emit_ids,
     ecs_iter_t *it,
     ecs_table_t *table,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
-    ecs_assert(cdr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
-    ecs_reachable_cache_t *rc = &cdr->pair->reachable;
+    ecs_assert(cr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
+    ecs_reachable_cache_t *rc = &cr->pair->reachable;
 
     if (rc->current != rc->generation) {
         /* Cache miss, iterate the tree to find ids to forward */
         if (ecs_should_log_3()) {
-            char *idstr = ecs_id_str(world, cdr->id);
+            char *idstr = ecs_id_str(world, cr->id);
             ecs_dbg_3("reachable cache miss for %s", idstr);
             ecs_os_free(idstr);
         }
@@ -1024,7 +1027,7 @@ void flecs_emit_forward(
         ecs_vec_init_t(&world->allocator, &stack, ecs_table_t*, 0);
         ecs_vec_reset_t(&world->allocator, &rc->ids, ecs_reachable_elem_t);
         flecs_emit_forward_up(world, er, er_onset, emit_ids, it, table, 
-            cdr, &stack, &rc->ids, 0);
+            cr, &stack, &rc->ids, 0);
         it->sources[0] = 0;
         ecs_vec_fini_t(&world->allocator, &stack, ecs_table_t*);
 
@@ -1046,13 +1049,13 @@ void flecs_emit_forward(
     } else {
         /* Cache hit, use cached values instead of walking the tree */
         if (ecs_should_log_3()) {
-            char *idstr = ecs_id_str(world, cdr->id);
+            char *idstr = ecs_id_str(world, cr->id);
             ecs_dbg_3("reachable cache hit for %s", idstr);
             ecs_os_free(idstr);
             flecs_emit_dump_cache(world, &rc->ids);
         }
 
-        ecs_entity_t trav = ECS_PAIR_FIRST(cdr->id);
+        ecs_entity_t trav = ECS_PAIR_FIRST(cr->id);
         ecs_reachable_elem_t *elems = ecs_vec_first_t(&rc->ids, 
             ecs_reachable_elem_t);
         int32_t i, count = ecs_vec_count(&rc->ids);
@@ -1060,17 +1063,17 @@ void flecs_emit_forward(
             ecs_reachable_elem_t *elem = &elems[i];
             const ecs_table_record_t *tr = elem->tr;
             ecs_assert(tr != NULL, ECS_INTERNAL_ERROR, NULL);
-            ecs_component_record_t *rc_idr = (ecs_component_record_t*)tr->hdr.cache;
+            ecs_component_record_t *rc_cr = (ecs_component_record_t*)tr->hdr.cache;
             ecs_record_t *r = elem->record;
 
-            ecs_assert(rc_idr->id == elem->id, ECS_INTERNAL_ERROR, NULL);
+            ecs_assert(rc_cr->id == elem->id, ECS_INTERNAL_ERROR, NULL);
             ecs_assert(r != NULL, ECS_INTERNAL_ERROR, NULL);
             ecs_assert(flecs_entities_get(world, elem->src) == r,
                 ECS_INTERNAL_ERROR, NULL);
             ecs_dbg_assert(r->table == elem->table, ECS_INTERNAL_ERROR, NULL);
 
             flecs_emit_forward_id(world, er, er_onset, emit_ids, it, table,
-                rc_idr, elem->src, r->table, tr->index, trav);
+                rc_cr, elem->src, r->table, tr->index, trav);
         }
     }
 
@@ -1081,7 +1084,7 @@ void flecs_emit_forward(
         entities = ECS_ELEM_T(entities, ecs_entity_t, it->offset);
         for (i = 0; i < it->count; i ++) {
             ecs_record_t *r = flecs_entities_get(world, entities[i]);
-            if (r->cdr) {
+            if (r->cr) {
                 break;
             }
         }
@@ -1094,10 +1097,10 @@ void flecs_emit_forward(
                 ecs_reachable_elem_t *elem = &elems[i];
                 const ecs_table_record_t *tr = elem->tr;
                 ecs_assert(tr != NULL, ECS_INTERNAL_ERROR, NULL);
-                ecs_component_record_t *rc_idr = (ecs_component_record_t*)tr->hdr.cache;
+                ecs_component_record_t *rc_cr = (ecs_component_record_t*)tr->hdr.cache;
                 ecs_record_t *r = elem->record;
 
-                ecs_assert(rc_idr->id == elem->id, ECS_INTERNAL_ERROR, NULL);
+                ecs_assert(rc_cr->id == elem->id, ECS_INTERNAL_ERROR, NULL);
                 ecs_assert(r != NULL, ECS_INTERNAL_ERROR, NULL);
                 ecs_assert(flecs_entities_get(world, elem->src) == r,
                     ECS_INTERNAL_ERROR, NULL);
@@ -1105,15 +1108,15 @@ void flecs_emit_forward(
                 (void)r;
 
                 /* If entities already have the component, don't propagate */
-                if (flecs_component_get_table(rc_idr, it->table)) {
+                if (flecs_component_get_table(rc_cr, it->table)) {
                     continue;
                 }
 
                 ecs_event_id_record_t *iders[5] = {0};
                 int32_t ider_count = flecs_event_observers_get(
-                    er, rc_idr->id, iders);
+                    er, rc_cr->id, iders);
 
-                flecs_propagate_entities(world, it, rc_idr, it->entities, 
+                flecs_propagate_entities(world, it, rc_cr, it->entities, 
                     it->count, elem->src, iders, ider_count);
             }
         }
@@ -1199,7 +1202,7 @@ void flecs_emit(
         .flags = desc->flags | EcsIterIsValid
     };
 
-    ecs_observable_t *observable = ecs_get_observable(desc->observable);
+    ecs_observable_t *observable = flecs_get_observable(desc->observable);
     ecs_check(observable != NULL, ECS_INVALID_PARAMETER, NULL);
 
     /* Event records contain all observers for a specific event. In addition to
@@ -1241,6 +1244,14 @@ void flecs_emit(
 
     ecs_event_id_record_t *iders[5] = {0};
 
+    /* Collect overridden ids so we can emit OnSet as last step */
+    int32_t override_count = 0;
+    ecs_id_t override_ids[256];
+    ecs_component_record_t *override_crs[256];
+    const ecs_table_record_t *override_trs[256];
+    ecs_size_t override_sizes[256];
+    ecs_table_record_t dummy_tr;
+
     if (count && can_forward && has_observed) {
         flecs_emit_propagate_invalidate(world, table, offset, count);
     }
@@ -1258,11 +1269,19 @@ repeat_event:
          * One example is when an id was added with a "With" property, or 
          * inheriting from a prefab with overrides. In these cases an entity is 
          * moved directly to the archetype with the additional components. */
-        ecs_component_record_t *cdr = NULL;
+        ecs_component_record_t *cr = NULL;
         const ecs_type_info_t *ti = NULL;
         ecs_id_t id = id_array[i];
-        ecs_assert(id == EcsAny || !ecs_id_is_wildcard(id), 
-            ECS_INVALID_PARAMETER, "cannot emit wildcard ids");
+
+        /* If id is wildcard this could be a remove(Rel, *) call for a 
+         * DontFragment component (for regular components this gets handled by
+         * the table graph which returns a vector with removed ids).
+         * This will be handled at a higher level than flecs_emit, so we can 
+         * ignore the wildcard */
+        if (id != EcsAny && ecs_id_is_wildcard(id)) {
+            continue;
+        }
+
         int32_t ider_i, ider_count = 0;
         bool is_pair = ECS_IS_PAIR(id);
         void *override_ptr = NULL;
@@ -1270,6 +1289,7 @@ repeat_event:
         ecs_table_record_t *base_tr = NULL;
         ecs_entity_t base = 0;
         bool id_can_override = can_override;
+        bool dont_fragment = false;
 
         /* Only added components can trigger overriding */
         if (set_mask && event == EcsOnAdd) {
@@ -1281,18 +1301,18 @@ repeat_event:
             }
         }
 
+        cr = flecs_components_get(world, id);
+        ecs_assert(cr != NULL, ECS_INTERNAL_ERROR, NULL);
+
+        ecs_flags32_t cr_flags = cr->flags;
+        dont_fragment = cr_flags & EcsIdDontFragment;
+
+        ti = cr->type_info;
+
         /* Check if this id is a pair of an traversable relationship. If so, we 
          * may have to forward ids from the pair's target. */
         if ((can_forward && is_pair) || id_can_override) {
-            cdr = flecs_components_get(world, id);
-            if (!cdr) {
-                /* Possible for union ids */
-                continue;
-            }
-
-            ecs_flags32_t idr_flags = cdr->flags;
-
-            if (is_pair && (idr_flags & EcsIdTraversable)) {
+            if (is_pair && (cr_flags & EcsIdTraversable)) {
                 const ecs_event_record_t *er_fwd = NULL;
                 if (ECS_PAIR_FIRST(id) == EcsIsA) {
                     if (event == EcsOnAdd) {
@@ -1307,7 +1327,15 @@ repeat_event:
                              * from being called recursively, in case prefab
                              * children also have IsA relationships. */
                             world->stages[0]->base = tgt;
-                            flecs_instantiate(world, tgt, table, offset, count, NULL);
+                            const ecs_entity_t *instances = 
+                                ecs_table_entities(table);
+                            int32_t e;
+
+                            for (e = 0; e < count; e ++) {
+                                flecs_instantiate(
+                                    world, tgt, instances[offset + e], NULL);
+                            }
+
                             world->stages[0]->base = 0;
                         }
 
@@ -1318,23 +1346,22 @@ repeat_event:
                 }
 
                 /* Forward events for components from pair target */
-                flecs_emit_forward(world, er, er_fwd, ids, &it, table, cdr);
+                flecs_emit_forward(world, er, er_fwd, ids, &it, table, cr);
                 ecs_assert(it.event_cur == evtx, ECS_INTERNAL_ERROR, NULL);
             }
 
-            if (id_can_override && !(idr_flags & EcsIdOnInstantiateDontInherit)) {
+            if (id_can_override && !(cr_flags & EcsIdOnInstantiateDontInherit)) {
                 /* Initialize overridden components with value from base */
-                ti = cdr->type_info;
                 if (ti) {
                     int32_t base_column = ecs_search_relation(world, table, 
                         0, id, EcsIsA, EcsUp, &base, NULL, &base_tr);
                     if (base_column != -1) {
                         /* Base found with component */
-                        ecs_table_t *base_table = base_tr->hdr.table;
-                        if (cdr->flags & EcsIdIsSparse) {
-                            override_ptr = flecs_sparse_get_any(
-                                cdr->sparse, 0, base);
+                        if (cr->flags & EcsIdIsSparse) {
+                            override_ptr = flecs_sparse_get(
+                                cr->sparse, 0, base);
                         } else {
+                            ecs_table_t *base_table = base_tr->hdr.table;
                             base_column = base_tr->column;
                             ecs_assert(base_column != -1, ECS_INTERNAL_ERROR, NULL);
                             ecs_record_t *base_r = flecs_entities_get(world, base);
@@ -1347,7 +1374,7 @@ repeat_event:
                         /* For ids with override policy, check if base was added 
                          * in same operation. This will determine later on 
                          * whether we need to emit an OnSet event. */
-                        if (!(cdr->flags & 
+                        if (!(cr->flags & 
                             (EcsIdOnInstantiateInherit|EcsIdOnInstantiateDontInherit))) {
                             int32_t base_i;
                             for (base_i = 0; base_i < id_count; base_i ++) {
@@ -1374,8 +1401,8 @@ repeat_event:
              * example, both observers for (ChildOf, p) and (ChildOf, *) would
              * match an event for (ChildOf, p). */
             ider_count = flecs_event_observers_get(er, id, iders);
-            cdr = cdr ? cdr : flecs_components_get(world, id);
-            ecs_assert(cdr != NULL, ECS_INTERNAL_ERROR, NULL);
+            cr = cr ? cr : flecs_components_get(world, id);
+            ecs_assert(cr != NULL, ECS_INTERNAL_ERROR, NULL);
         }
 
         if (!ider_count && !override_ptr) {
@@ -1383,17 +1410,17 @@ repeat_event:
             continue;
         }
 
-        ecs_assert(cdr != NULL, ECS_INTERNAL_ERROR, NULL);
-        const ecs_table_record_t *tr = flecs_component_get_table(cdr, table);
-        ecs_table_record_t dummy_tr = {
-            .hdr.cache = (ecs_table_cache_t*)cdr,
+        ecs_assert(cr != NULL, ECS_INTERNAL_ERROR, NULL);
+        const ecs_table_record_t *tr = flecs_component_get_table(cr, table);
+        dummy_tr = (ecs_table_record_t){
+            .hdr.cache = (ecs_table_cache_t*)cr,
             .hdr.table = table,
             .index = -1,
             .column = -1,
             .count = 0
         };
 
-        if (id != EcsAny) {
+        if (!dont_fragment && id != EcsAny) {
             if (tr == NULL) {
                 /* When a single batch contains multiple add's for an exclusive
                 * relationship, it's possible that an id was in the added list
@@ -1413,20 +1440,20 @@ repeat_event:
 
         if (count) {
             storage_i = tr->column;
-            bool is_sparse = cdr->flags & EcsIdIsSparse;
 
-            if (!ecs_id_is_wildcard(id) && (storage_i != -1 || is_sparse)) {
+            if (!ecs_id_is_wildcard(id) && (storage_i != -1 || ti)) {
+                bool is_sparse = cr->flags & EcsIdIsSparse;
+                ecs_size_t size = ti->size;
                 void *ptr;
-                ecs_size_t size = cdr->type_info->size;
 
                 if (is_sparse) {
                     ecs_assert(count == 1, ECS_UNSUPPORTED, 
                         "events for multiple entities are currently unsupported"
                         " for sparse components");
                     ecs_entity_t e = ecs_table_entities(table)[offset];
-                    ptr = flecs_sparse_get(cdr->sparse, 0, e);
+                    ptr = flecs_sparse_get(cr->sparse, 0, e);
                 } else{
-                    ecs_assert(cdr->type_info != NULL, ECS_INTERNAL_ERROR, NULL);
+                    ecs_assert(cr->type_info != NULL, ECS_INTERNAL_ERROR, NULL);
                     ecs_column_t *c = &columns[storage_i];
                     ptr = ECS_ELEM(c->data, size, offset);
                 }
@@ -1445,28 +1472,15 @@ repeat_event:
                          * operation, generate an OnSet event as this is the 
                          * first time this value is observed for the entity. */
                         if (override_base_added) {
-                            ecs_event_id_record_t *iders_set[5] = {0};
-                            int32_t ider_set_i, ider_set_count = 
-                                flecs_event_observers_get(er_onset, id, iders_set);
-                            for (ider_set_i = 0; ider_set_i < ider_set_count; ider_set_i ++) {
-                                ecs_event_id_record_t *ider = iders_set[ider_set_i];
-                                flecs_observers_invoke(
-                                    world, &ider->self, &it, table, 0);
-                                ecs_assert(it.event_cur == evtx, 
-                                    ECS_INTERNAL_ERROR, NULL);
-                                flecs_observers_invoke(
-                                    world, &ider->self_up, &it, table, 0);
-                                ecs_assert(it.event_cur == evtx, 
-                                    ECS_INTERNAL_ERROR, NULL);
-                            }
+                            override_ids[override_count] = id;
+                            override_crs[override_count] = cr;
+                            override_trs[override_count] = tr;
+                            override_sizes[override_count] = size;
+                            override_count ++;
                         }
                     } else if (er_onset && it.other_table && 
                             it.other_table->type.count) 
                     {
-                        /* If an override was removed, this re-exposes the
-                         * overridden component. Because this causes the actual
-                         * (now inherited) value of the component to change, an
-                         * OnSet event must be emitted for the base component.*/
                         ecs_assert(event == EcsOnRemove, ECS_INTERNAL_ERROR, NULL);
                         ecs_event_id_record_t *iders_set[5] = {0};
                         int32_t ider_set_i, ider_set_count = 
@@ -1516,7 +1530,7 @@ repeat_event:
          * entity/entities for which the event was generated is used as such a
          * target, events must be propagated downwards. */
         flecs_propagate_entities(
-            world, &it, cdr, it.entities, count, 0, iders, ider_count);
+            world, &it, cr, it.entities, count, 0, iders, ider_count);
     }
 
     can_override = false; /* Don't override twice */
@@ -1529,6 +1543,50 @@ repeat_event:
         goto repeat_event;
     }
 
+    if (override_count) {
+        it.sources[0] = 0;
+        it.up_fields = 0;
+
+        for (i = 0; i < override_count; i ++) {
+            ecs_id_t id = override_ids[i];
+            const ecs_table_record_t *tr = override_trs[i];
+
+            if (tr == &dummy_tr) {
+                dummy_tr = (ecs_table_record_t){
+                    .hdr.cache = (ecs_table_cache_t*)override_crs[i],
+                    .hdr.table = table,
+                    .index = -1,
+                    .column = -1,
+                    .count = 0
+                };
+            }
+
+            it.trs[0] = tr;
+            ECS_CONST_CAST(int32_t*, it.sizes)[0] = override_sizes[i];
+            it.event_id = id;
+            it.ids[0] = id;
+
+            ecs_event_id_record_t *iders_set[5] = {0};
+            int32_t ider_set_i, ider_set_count = 
+                flecs_event_observers_get(er_onset, id, iders_set);
+            if (!ider_set_count) {
+                continue;
+            }
+
+            for (ider_set_i = 0; ider_set_i < ider_set_count; ider_set_i ++) {
+                ecs_event_id_record_t *ider = iders_set[ider_set_i];
+                flecs_observers_invoke(
+                    world, &ider->self, &it, table, 0);
+                ecs_assert(it.event_cur == evtx, 
+                    ECS_INTERNAL_ERROR, NULL);
+                flecs_observers_invoke(
+                    world, &ider->self_up, &it, table, 0);
+                ecs_assert(it.event_cur == evtx, 
+                    ECS_INTERNAL_ERROR, NULL);
+            }
+        }
+    }
+
 error:
     world->stages[0]->defer = defer;
 
