@@ -1087,29 +1087,20 @@ void flecs_query_set_op_kind(
     } else if (term->flags_ & EcsTermTransitive) {
         ecs_assert(ecs_term_ref_is_set(&term->second), ECS_INTERNAL_ERROR, NULL);
         op->kind = EcsQueryTrav;
-
-    /* If term queries for union pair, use union instruction */
-    } else if (term->flags_ & EcsTermIsUnion) {
+    } else if (term->flags_ & EcsTermDontFragment) {
         if (op->kind == EcsQueryAnd) {
-            op->kind = EcsQueryUnionEq;
+            op->kind = EcsQuerySparse;
             if (term->oper == EcsNot) {
-                if (!ecs_id_is_wildcard(ECS_TERM_REF_ID(&term->second))) {
-                    term->oper = EcsAnd;
-                    op->kind = EcsQueryUnionNeq;
-                }
+                op->kind = EcsQuerySparseNot;
             }
         } else {
-            op->kind = EcsQueryUnionEqWith;
+            op->kind = EcsQuerySparseWith;
         }
 
         if ((term->src.id & trav_flags) == EcsUp) {
-            if (op->kind == EcsQueryUnionEq) {
-                op->kind = EcsQueryUnionEqUp;
-            }
+            op->kind = EcsQuerySparseUp;
         } else if ((term->src.id & trav_flags) == (EcsSelf|EcsUp)) {
-            if (op->kind == EcsQueryUnionEq) {
-                op->kind = EcsQueryUnionEqSelfUp;
-            }
+            op->kind = EcsQuerySparseSelfUp;
         }
     } else {
         if ((term->src.id & trav_flags) == EcsUp) {
@@ -1118,6 +1109,14 @@ void flecs_query_set_op_kind(
             op->kind = EcsQuerySelfUp;
         } else if (term->flags_ & (EcsTermMatchAny|EcsTermMatchAnySrc)) {
             op->kind = EcsQueryAndAny;
+        } else if (ECS_IS_PAIR(term->id) && 
+            ECS_PAIR_FIRST(term->id) == EcsWildcard) 
+        {
+            if (op->kind == EcsQueryAnd) {
+                op->kind = EcsQueryAndWcTgt;
+            } else {
+                op->kind = EcsQueryWithWcTgt;
+            }
         }
     }
 }
@@ -1173,8 +1172,8 @@ int flecs_query_compile_term(
 
         for (i = 0; i < count; i ++) {
             ecs_id_t ti_id = ti_ids[i];
-            ecs_component_record_t *cdr = flecs_components_get(world, ti_id);
-            if (!(cdr->flags & EcsIdOnInstantiateDontInherit)) {
+            ecs_component_record_t *cr = flecs_components_get(world, ti_id);
+            if (!(cr->flags & EcsIdOnInstantiateDontInherit)) {
                 break;
             }
         }
@@ -1221,6 +1220,9 @@ int flecs_query_compile_term(
     flecs_query_set_op_kind(&op, term, src_is_var);
 
     bool is_not = (term->oper == EcsNot) && !builtin_pred;
+    if (op.kind == EcsQuerySparseNot) {
+        is_not = false;
+    }
 
     /* Save write state at start of term so we can use it to reliably track
      * variables got written by this term. */
