@@ -25,24 +25,24 @@ bool flecs_query_select_w_id(
     ecs_flags32_t filter_mask)
 {
     ecs_query_and_ctx_t *op_ctx = flecs_op_ctx(ctx, and);
-    ecs_component_record_t *cdr = op_ctx->cdr;
+    ecs_component_record_t *cr = op_ctx->cr;
     const ecs_table_record_t *tr;
     ecs_table_t *table;
 
     if (!redo) {
-        if (!cdr || cdr->id != id) {
-            cdr = op_ctx->cdr = flecs_components_get(ctx->world, id);
-            if (!cdr) {
+        if (!cr || cr->id != id) {
+            cr = op_ctx->cr = flecs_components_get(ctx->world, id);
+            if (!cr) {
                 return false;
             }
         }
 
         if (ctx->query->pub.flags & EcsQueryMatchEmptyTables) {
-            if (!flecs_table_cache_all_iter(&cdr->cache, &op_ctx->it)) {
+            if (!flecs_table_cache_all_iter(&cr->cache, &op_ctx->it)) {
                 return false;
             }
         } else {
-            if (!flecs_table_cache_iter(&cdr->cache, &op_ctx->it)) {
+            if (!flecs_table_cache_iter(&cr->cache, &op_ctx->it)) {
                 return false;
             }
         }
@@ -63,7 +63,7 @@ repeat:
         tr = (const ecs_table_record_t*)op_ctx->it.cur;
         ecs_assert(tr != NULL, ECS_INTERNAL_ERROR, NULL);
         table = tr->hdr.table;
-        op_ctx->column = flecs_query_next_column(table, cdr->id, op_ctx->column);
+        op_ctx->column = flecs_query_next_column(table, cr->id, op_ctx->column);
         op_ctx->remaining --;
     }
 
@@ -94,7 +94,7 @@ bool flecs_query_with(
     const ecs_query_run_ctx_t *ctx)
 {
     ecs_query_and_ctx_t *op_ctx = flecs_op_ctx(ctx, and);
-    ecs_component_record_t *cdr = op_ctx->cdr;
+    ecs_component_record_t *cr = op_ctx->cr;
     const ecs_table_record_t *tr;
 
     ecs_table_t *table = flecs_query_get_table(op, &op->src, EcsQuerySrc, ctx);
@@ -104,14 +104,14 @@ bool flecs_query_with(
 
     if (!redo) {
         ecs_id_t id = flecs_query_op_get_id(op, ctx);
-        if (!cdr || cdr->id != id) {
-            cdr = op_ctx->cdr = flecs_components_get(ctx->world, id);
-            if (!cdr) {
+        if (!cr || cr->id != id) {
+            cr = op_ctx->cr = flecs_components_get(ctx->world, id);
+            if (!cr) {
                 return false;
             }
         }
 
-        tr = flecs_component_get_table(cdr, table);
+        tr = flecs_component_get_table(cr, table);
         if (!tr) {
             return false;
         }
@@ -127,7 +127,7 @@ bool flecs_query_with(
             return false;
         }
 
-        op_ctx->column = flecs_query_next_column(table, cdr->id, op_ctx->column);
+        op_ctx->column = flecs_query_next_column(table, cr->id, op_ctx->column);
         ecs_assert(op_ctx->column != -1, ECS_INTERNAL_ERROR, NULL);
     }
 
@@ -225,20 +225,20 @@ bool flecs_query_select_id(
 
     if (!redo) {
         ecs_id_t id = it->ids[field];
-        ecs_component_record_t *cdr = op_ctx->cdr;
-        if (!cdr || cdr->id != id) {
-            cdr = op_ctx->cdr = flecs_components_get(ctx->world, id);
-            if (!cdr) {
+        ecs_component_record_t *cr = op_ctx->cr;
+        if (!cr || cr->id != id) {
+            cr = op_ctx->cr = flecs_components_get(ctx->world, id);
+            if (!cr) {
                 return false;
             }
         }
 
         if (ctx->query->pub.flags & EcsQueryMatchEmptyTables) {
-            if (!flecs_table_cache_all_iter(&cdr->cache, &op_ctx->it)) {
+            if (!flecs_table_cache_all_iter(&cr->cache, &op_ctx->it)) {
                 return false;
             }
         } else {
-            if (!flecs_table_cache_iter(&cdr->cache, &op_ctx->it)) {
+            if (!flecs_table_cache_iter(&cr->cache, &op_ctx->it)) {
                 return false;
             }
         }
@@ -279,15 +279,15 @@ bool flecs_query_with_id(
     ecs_assert(table != NULL, ECS_INTERNAL_ERROR, NULL);
 
     ecs_id_t id = it->ids[field];
-    ecs_component_record_t *cdr = op_ctx->cdr;
-    if (!cdr || cdr->id != id) {
-        cdr = op_ctx->cdr = flecs_components_get(ctx->world, id);
-        if (!cdr) {
+    ecs_component_record_t *cr = op_ctx->cr;
+    if (!cr || cr->id != id) {
+        cr = op_ctx->cr = flecs_components_get(ctx->world, id);
+        if (!cr) {
             return false;
         }
     }
 
-    const ecs_table_record_t *tr = flecs_component_get_table(cdr, table);
+    const ecs_table_record_t *tr = flecs_component_get_table(cr, table);
     if (!tr) {
         return false;
     }
@@ -365,6 +365,78 @@ bool flecs_query_and_any(
     return result;
 }
 
+static
+bool flecs_query_and_wctgt(
+    const ecs_query_op_t *op,
+    bool redo,
+    const ecs_query_run_ctx_t *ctx)
+{
+    ecs_query_and_ctx_t *op_ctx = flecs_op_ctx(ctx, and);
+    if (!redo) {
+        op_ctx->non_fragmenting = false;
+    }
+    
+    bool sparse_redo = true;
+
+    if (!op_ctx->non_fragmenting) {
+        bool result = flecs_query_and(op, redo, ctx);
+        if (result) {
+            return true;
+        }
+
+        ecs_component_record_t *cr = op_ctx->cr;
+
+        if (!cr) {
+            return false;
+        }
+
+        if (!(cr->flags & EcsIdMatchDontFragment) && 
+             (cr->id != ecs_pair(EcsWildcard, EcsWildcard))) 
+        {
+            return false;
+        }
+
+        op_ctx->non_fragmenting = true;
+        sparse_redo = false;
+    }
+
+    return flecs_query_sparse(op, sparse_redo, ctx);
+}
+
+static
+bool flecs_query_with_wctgt(
+    const ecs_query_op_t *op,
+    bool redo,
+    const ecs_query_run_ctx_t *ctx)
+{
+    ecs_query_and_ctx_t *op_ctx = flecs_op_ctx(ctx, and);
+    if (!redo) {
+        op_ctx->non_fragmenting = false;
+    }
+    
+    bool sparse_redo = true;
+
+    if (!op_ctx->non_fragmenting) {
+        bool result = flecs_query_with(op, redo, ctx);
+        if (result) {
+            return true;
+        }
+
+        if (!op_ctx->cr) {
+            return false;
+        }
+
+        if (!(op_ctx->cr->flags & EcsIdMatchDontFragment)) {
+            return false;
+        }
+
+        op_ctx->non_fragmenting = true;
+        sparse_redo = false;
+    }
+
+    return flecs_query_sparse_with(op, sparse_redo, ctx, false);
+}
+
 static
 bool flecs_query_triv(
     const ecs_query_op_t *op,
@@ -426,8 +498,8 @@ int32_t flecs_query_next_inheritable_id(
 {
     int32_t i;
     for (i = index; i < type->count; i ++) {
-        ecs_component_record_t *cdr = flecs_components_get(world, type->array[i]);
-        if (!(cdr->flags & EcsIdOnInstantiateDontInherit)) {
+        ecs_component_record_t *cr = flecs_components_get(world, type->array[i]);
+        if (!(cr->flags & EcsIdOnInstantiateDontInherit)) {
             return i;
         }
     }
@@ -521,16 +593,16 @@ bool flecs_query_x_from(
              * components from the type list */
             if (op_ctx->cur_id_index != op_ctx->first_id_index) {
                 for (i = op_ctx->first_id_index; i < op_ctx->cur_id_index; i ++) {
-                    ecs_component_record_t *cdr = flecs_components_get(world, ids[i]);
-                    if (!cdr) {
+                    ecs_component_record_t *cr = flecs_components_get(world, ids[i]);
+                    if (!cr) {
                         continue;
                     }
 
-                    if (cdr->flags & EcsIdOnInstantiateDontInherit) {
+                    if (cr->flags & EcsIdOnInstantiateDontInherit) {
                         continue;
                     }
                     
-                    if (flecs_component_get_table(cdr, src_table) != NULL) {
+                    if (flecs_component_get_table(cr, src_table) != NULL) {
                         /* Already matched */
                         break;
                     }
@@ -544,8 +616,8 @@ bool flecs_query_x_from(
 
         if (oper == EcsAndFrom || oper == EcsNotFrom || src_written) {
             for (i = id_index; i < type->count; i ++) {
-                ecs_component_record_t *cdr = flecs_components_get(world, ids[i]);
-                if (!cdr) {
+                ecs_component_record_t *cr = flecs_components_get(world, ids[i]);
+                if (!cr) {
                     if (oper == EcsAndFrom) {
                         return false;
                     } else {
@@ -553,11 +625,11 @@ bool flecs_query_x_from(
                     }
                 }
 
-                if (cdr->flags & EcsIdOnInstantiateDontInherit) {
+                if (cr->flags & EcsIdOnInstantiateDontInherit) {
                     continue;
                 }
 
-                if (flecs_component_get_table(cdr, src_table) == NULL) {
+                if (flecs_component_get_table(cr, src_table) == NULL) {
                     if (oper == EcsAndFrom) {
                         break; /* Must have all ids */
                     }
@@ -1392,12 +1464,14 @@ bool flecs_query_dispatch(
     case EcsQueryAll: return flecs_query_all(op, redo, ctx);
     case EcsQueryAnd: return flecs_query_and(op, redo, ctx);
     case EcsQueryAndAny: return flecs_query_and_any(op, redo, ctx);
+    case EcsQueryAndWcTgt: return flecs_query_and_wctgt(op, redo, ctx);
     case EcsQueryTriv: return flecs_query_triv(op, redo, ctx);
     case EcsQueryCache: return flecs_query_cache(op, redo, ctx);
     case EcsQueryIsCache: return flecs_query_is_cache(op, redo, ctx);
     case EcsQueryUp: return flecs_query_up(op, redo, ctx);
     case EcsQuerySelfUp: return flecs_query_self_up(op, redo, ctx);
     case EcsQueryWith: return flecs_query_with(op, redo, ctx);
+    case EcsQueryWithWcTgt: return flecs_query_with_wctgt(op, redo, ctx);
     case EcsQueryTrav: return flecs_query_trav(op, redo, ctx);
     case EcsQueryAndFrom: return flecs_query_and_from(op, redo, ctx);
     case EcsQueryNotFrom: return flecs_query_not_from(op, redo, ctx);
@@ -1424,11 +1498,11 @@ bool flecs_query_dispatch(
     case EcsQueryMemberNeq: return flecs_query_member_neq(op, redo, ctx);
     case EcsQueryToggle: return flecs_query_toggle(op, redo, ctx);
     case EcsQueryToggleOption: return flecs_query_toggle_option(op, redo, ctx);
-    case EcsQueryUnionEq: return flecs_query_union(op, redo, ctx);
-    case EcsQueryUnionEqWith: return flecs_query_union_with(op, redo, ctx, false);
-    case EcsQueryUnionNeq: return flecs_query_union_neq(op, redo, ctx);
-    case EcsQueryUnionEqUp: return flecs_query_union_up(op, redo, ctx);
-    case EcsQueryUnionEqSelfUp: return flecs_query_union_self_up(op, redo, ctx);
+    case EcsQuerySparse: return flecs_query_sparse(op, redo, ctx);
+    case EcsQuerySparseWith: return flecs_query_sparse_with(op, redo, ctx, false);
+    case EcsQuerySparseNot: return flecs_query_sparse_with(op, redo, ctx, true);
+    case EcsQuerySparseSelfUp: return flecs_query_sparse_self_up(op, redo, ctx);
+    case EcsQuerySparseUp: return flecs_query_sparse_up(op, redo, ctx);
     case EcsQueryLookup: return flecs_query_lookup(op, redo, ctx);
     case EcsQuerySetVars: return flecs_query_setvars(op, redo, ctx);
     case EcsQuerySetThis: return flecs_query_setthis(op, redo, ctx);
