@@ -20,41 +20,41 @@ static
 ecs_id_record_elem_t* flecs_component_elem(
     ecs_component_record_t *head,
     ecs_id_record_elem_t *list,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
-    return ECS_OFFSET(cdr->pair, (uintptr_t)list - (uintptr_t)head->pair);
+    return ECS_OFFSET(cr->pair, (uintptr_t)list - (uintptr_t)head->pair);
 }
 
 static
 void flecs_component_elem_insert(
     ecs_component_record_t *head,
-    ecs_component_record_t *cdr,
+    ecs_component_record_t *cr,
     ecs_id_record_elem_t *elem)
 {
-    ecs_id_record_elem_t *head_elem = flecs_component_elem(cdr, elem, head);
+    ecs_id_record_elem_t *head_elem = flecs_component_elem(cr, elem, head);
     ecs_component_record_t *cur = head_elem->next;
     elem->next = cur;
     elem->prev = head;
     if (cur) {
-        ecs_id_record_elem_t *cur_elem = flecs_component_elem(cdr, elem, cur);
-        cur_elem->prev = cdr;
+        ecs_id_record_elem_t *cur_elem = flecs_component_elem(cr, elem, cur);
+        cur_elem->prev = cr;
     }
-    head_elem->next = cdr;
+    head_elem->next = cr;
 }
 
 static
 void flecs_component_elem_remove(
-    ecs_component_record_t *cdr,
+    ecs_component_record_t *cr,
     ecs_id_record_elem_t *elem)
 {
     ecs_component_record_t *prev = elem->prev;
     ecs_component_record_t *next = elem->next;
     ecs_assert(prev != NULL, ECS_INTERNAL_ERROR, NULL);
 
-    ecs_id_record_elem_t *prev_elem = flecs_component_elem(cdr, elem, prev);
+    ecs_id_record_elem_t *prev_elem = flecs_component_elem(cr, elem, prev);
     prev_elem->next = next;
     if (next) {
-        ecs_id_record_elem_t *next_elem = flecs_component_elem(cdr, elem, next);
+        ecs_id_record_elem_t *next_elem = flecs_component_elem(cr, elem, next);
         next_elem->prev = prev;
     }
 }
@@ -62,55 +62,55 @@ void flecs_component_elem_remove(
 static
 void flecs_insert_id_elem(
     ecs_world_t *world,
-    ecs_component_record_t *cdr,
+    ecs_component_record_t *cr,
     ecs_id_t wildcard,
-    ecs_component_record_t *widr)
+    ecs_component_record_t *wcr)
 {
     ecs_assert(ecs_id_is_wildcard(wildcard), ECS_INTERNAL_ERROR, NULL);
-    if (!widr) {
-        widr = flecs_components_ensure(world, wildcard);
+    if (!wcr) {
+        wcr = flecs_components_ensure(world, wildcard);
     }
-    ecs_assert(widr != NULL, ECS_INTERNAL_ERROR, NULL);
+    ecs_assert(wcr != NULL, ECS_INTERNAL_ERROR, NULL);
 
-    ecs_pair_id_record_t *pair = cdr->pair;
+    ecs_pair_record_t *pair = cr->pair;
     ecs_assert(pair != NULL, ECS_INTERNAL_ERROR, NULL);
 
     if (ECS_PAIR_SECOND(wildcard) == EcsWildcard) {
         ecs_assert(ECS_PAIR_FIRST(wildcard) != EcsWildcard, 
             ECS_INTERNAL_ERROR, NULL);
-        flecs_component_elem_insert(widr, cdr, &pair->first);
+        flecs_component_elem_insert(wcr, cr, &pair->first);
     } else {
         ecs_assert(ECS_PAIR_FIRST(wildcard) == EcsWildcard, 
             ECS_INTERNAL_ERROR, NULL);
-        flecs_component_elem_insert(widr, cdr, &pair->second);
+        flecs_component_elem_insert(wcr, cr, &pair->second);
 
-        if (cdr->flags & EcsIdTraversable) {
-            flecs_component_elem_insert(widr, cdr, &pair->trav);
+        if (cr->flags & EcsIdTraversable) {
+            flecs_component_elem_insert(wcr, cr, &pair->trav);
         }
     }
 }
 
 static
 void flecs_remove_id_elem(
-    ecs_component_record_t *cdr,
+    ecs_component_record_t *cr,
     ecs_id_t wildcard)
 {
     ecs_assert(ecs_id_is_wildcard(wildcard), ECS_INTERNAL_ERROR, NULL);
 
-    ecs_pair_id_record_t *pair = cdr->pair;
+    ecs_pair_record_t *pair = cr->pair;
     ecs_assert(pair != NULL, ECS_INTERNAL_ERROR, NULL);
 
     if (ECS_PAIR_SECOND(wildcard) == EcsWildcard) {
         ecs_assert(ECS_PAIR_FIRST(wildcard) != EcsWildcard, 
             ECS_INTERNAL_ERROR, NULL);
-        flecs_component_elem_remove(cdr, &pair->first);
+        flecs_component_elem_remove(cr, &pair->first);
     } else {
         ecs_assert(ECS_PAIR_FIRST(wildcard) == EcsWildcard, 
             ECS_INTERNAL_ERROR, NULL);
-        flecs_component_elem_remove(cdr, &pair->second);
+        flecs_component_elem_remove(cr, &pair->second);
 
-        if (cdr->flags & EcsIdTraversable) {
-            flecs_component_elem_remove(cdr, &pair->trav);
+        if (cr->flags & EcsIdTraversable) {
+            flecs_component_elem_remove(cr, &pair->trav);
         }
     }
 }
@@ -136,42 +136,100 @@ ecs_id_t flecs_component_hash(
 
 void flecs_component_init_sparse(
     ecs_world_t *world,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
-    if (!cdr->sparse) {
-        if (cdr->flags & EcsIdIsSparse) {
-            ecs_assert(!(cdr->flags & EcsIdIsUnion), ECS_CONSTRAINT_VIOLATED,
-                "cannot mix union and sparse traits");
-            ecs_assert(cdr->type_info != NULL, ECS_INVALID_OPERATION, 
-                "only components can be marked as sparse");
-            cdr->sparse = flecs_walloc_t(world, ecs_sparse_t);
-            flecs_sparse_init(cdr->sparse, NULL, NULL, cdr->type_info->size);
-        } else
-        if (cdr->flags & EcsIdIsUnion) {
-            cdr->sparse = flecs_walloc_t(world, ecs_switch_t);
-            flecs_switch_init(cdr->sparse, &world->allocator);
+    if (!ecs_id_is_wildcard(cr->id)) {
+        if (!cr->sparse) {
+            if (cr->flags & EcsIdIsSparse) {
+                cr->sparse = flecs_walloc_t(world, ecs_sparse_t);
+                if (cr->type_info) {
+                    flecs_sparse_init(cr->sparse, NULL, NULL, cr->type_info->size);
+                } else {
+                    flecs_sparse_init(cr->sparse, NULL, NULL, 0);
+                }
+            }
+        }
+
+        if (cr->id < FLECS_HI_COMPONENT_ID) {
+            world->non_trivial[cr->id] |= EcsNonTrivialIdSparse;
+        }
+    } else if (ECS_IS_PAIR(cr->id)) {
+        if (cr->flags & EcsIdDontFragment) {
+            if (cr->sparse) {
+                ecs_assert(flecs_sparse_count(cr->sparse) == 0,
+                    ECS_INTERNAL_ERROR, NULL);
+                flecs_sparse_fini(cr->sparse);
+            } else {
+                cr->sparse = flecs_walloc_t(world, ecs_sparse_t);
+            }
+
+            if (cr->flags & EcsIdExclusive) {
+                flecs_sparse_init_t(cr->sparse, NULL, NULL, ecs_entity_t);
+            } else {
+                flecs_sparse_init_t(cr->sparse, NULL, NULL, ecs_type_t);
+            }
         }
     }
 }
 
+void flecs_component_record_init_dont_fragment(
+    ecs_world_t *world,
+    ecs_component_record_t *cr)
+{
+    if (world->cr_non_fragmenting_head) {
+        world->cr_non_fragmenting_head->non_fragmenting.prev = cr;
+    }
+
+    cr->non_fragmenting.next = world->cr_non_fragmenting_head;
+    world->cr_non_fragmenting_head = cr;
+
+    if (cr->id < FLECS_HI_COMPONENT_ID) {
+        world->non_trivial[cr->id] |= EcsNonTrivialIdNonFragmenting;
+    }
+
+    flecs_component_init_sparse(world, cr);
+}
+
+static
+void flecs_component_record_fini_dont_fragment(
+    ecs_world_t *world,
+    ecs_component_record_t *cr)
+{
+    if (world->cr_non_fragmenting_head == cr) {
+        world->cr_non_fragmenting_head = cr->non_fragmenting.next;
+    }
+
+    if (cr->non_fragmenting.prev) {
+        cr->non_fragmenting.prev->non_fragmenting.next = 
+            cr->non_fragmenting.next;
+    }
+
+    if (cr->non_fragmenting.next) {
+        cr->non_fragmenting.next->non_fragmenting.prev = 
+            cr->non_fragmenting.prev;
+    }
+}
+
+void flecs_component_record_init_exclusive(
+    ecs_world_t *world,
+    ecs_component_record_t *cr)
+{
+    flecs_component_init_sparse(world, cr);
+}
+
 static
 void flecs_component_fini_sparse(
     ecs_world_t *world,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
-    if (cdr->sparse) {
-        if (cdr->flags & EcsIdIsSparse) {
-            ecs_assert(flecs_sparse_count(cdr->sparse) == 0, 
-                ECS_INTERNAL_ERROR, NULL);
-            flecs_sparse_fini(cdr->sparse);
-            flecs_wfree_t(world, ecs_sparse_t, cdr->sparse);
-        } else
-        if (cdr->flags & EcsIdIsUnion) {
-            flecs_switch_fini(cdr->sparse);
-            flecs_wfree_t(world, ecs_switch_t, cdr->sparse);
-        } else {
-            ecs_abort(ECS_INTERNAL_ERROR, "unknown sparse storage");
+    if (cr->sparse) {
+        ecs_assert(cr->flags & EcsIdIsSparse, ECS_INTERNAL_ERROR, NULL);
+        if (cr->flags & EcsIdDontFragment) {
+            flecs_component_sparse_remove_all(world, cr);
         }
+
+        flecs_sparse_fini(cr->sparse);
+        flecs_wfree_t(world, ecs_sparse_t, cr->sparse);
     }
 }
 
@@ -195,30 +253,32 @@ ecs_component_record_t* flecs_component_new(
     ecs_world_t *world,
     ecs_id_t id)
 {
-    ecs_component_record_t *cdr, *idr_t = NULL;
+    ecs_component_record_t *cr, *cr_t = NULL;
     ecs_id_t hash = flecs_component_hash(id);
-    cdr = flecs_bcalloc_w_dbg_info(
+    cr = flecs_bcalloc_w_dbg_info(
         &world->allocators.id_record, "ecs_component_record_t");
 
     if (hash >= FLECS_HI_ID_RECORD_ID) {
-        ecs_map_insert_ptr(&world->id_index_hi, hash, cdr);
+        ecs_map_insert_ptr(&world->id_index_hi, hash, cr);
     } else {
-        world->id_index_lo[hash] = cdr;
+        world->id_index_lo[hash] = cr;
     }
 
-    ecs_table_cache_init(world, &cdr->cache);
+    ecs_table_cache_init(world, &cr->cache);
 
-    cdr->id = id;
-    cdr->refcount = 1;
+    cr->id = id;
+    cr->refcount = 1;
 
     bool is_wildcard = ecs_id_is_wildcard(id);
     bool is_pair = ECS_IS_PAIR(id);
 
     ecs_entity_t rel = 0, tgt = 0, role = id & ECS_ID_FLAGS_MASK;
     if (is_pair) {
-        cdr->pair = flecs_bcalloc_w_dbg_info(
-            &world->allocators.pair_id_record, "ecs_pair_id_record_t");
-        cdr->pair->reachable.current = -1;
+        cr->pair = flecs_bcalloc_w_dbg_info(
+            &world->allocators.pair_id_record, "ecs_pair_record_t");
+        cr->pair->reachable.current = -1;
+
+        flecs_ordered_children_init(world, cr);
 
         rel = ECS_PAIR_FIRST(id);
         rel = flecs_entities_get_alive(world, rel);
@@ -265,7 +325,7 @@ ecs_component_record_t* flecs_component_new(
             #endif
         }
 
-        if (tgt && !ecs_id_is_wildcard(tgt) && tgt != EcsUnion) {
+        if (tgt && !ecs_id_is_wildcard(tgt)) {
             /* Check if target of relationship satisfies OneOf property */
             ecs_entity_t oneof = flecs_get_oneof(world, rel);
             if (oneof) {
@@ -305,18 +365,18 @@ ecs_component_record_t* flecs_component_new(
 
         if (!is_wildcard && (rel != EcsFlag)) {
             /* Inherit flags from (relationship, *) record */
-            ecs_component_record_t *idr_r = flecs_components_ensure(
+            ecs_component_record_t *cr_r = flecs_components_ensure(
                 world, ecs_pair(rel, EcsWildcard));
-            cdr->pair->parent = idr_r;
-            cdr->flags = idr_r->flags;
+            cr->pair->parent = cr_r;
+            cr->flags = cr_r->flags;
 
             /* If pair is not a wildcard, append it to wildcard lists. These 
              * allow for quickly enumerating all relationships for an object, 
              * or all objects for a relationship. */
-            flecs_insert_id_elem(world, cdr, ecs_pair(rel, EcsWildcard), idr_r);
+            flecs_insert_id_elem(world, cr, ecs_pair(rel, EcsWildcard), cr_r);
 
-            idr_t = flecs_components_ensure(world, ecs_pair(EcsWildcard, tgt));
-            flecs_insert_id_elem(world, cdr, ecs_pair(EcsWildcard, tgt), idr_t);
+            cr_t = flecs_components_ensure(world, ecs_pair(EcsWildcard, tgt));
+            flecs_insert_id_elem(world, cr, ecs_pair(EcsWildcard, tgt), cr_t);
         }
     } else {
         rel = id & ECS_COMPONENT_MASK;
@@ -345,12 +405,12 @@ ecs_component_record_t* flecs_component_new(
 
     /* Initialize type info if id is not a tag */
     if (!is_wildcard && (!role || is_pair)) {
-        if (!(cdr->flags & EcsIdTag)) {
+        if (!(cr->flags & EcsIdTag)) {
             const ecs_type_info_t *ti = flecs_type_info_get(world, rel);
             if (!ti && tgt) {
                 ti = flecs_type_info_get(world, tgt);
             }
-            cdr->type_info = ti;
+            cr->type_info = ti;
         }
     }
 
@@ -365,32 +425,50 @@ ecs_component_record_t* flecs_component_new(
         ecs_record_t *tgt_r = flecs_entities_get_any(world, tgt);
         ecs_assert(tgt_r != NULL, ECS_INTERNAL_ERROR, NULL);
         flecs_record_add_flag(tgt_r, EcsEntityIsTarget);
-        if (cdr->flags & EcsIdTraversable) {
+        if (cr->flags & EcsIdTraversable) {
             /* Flag used to determine if object should be traversed when
              * propagating events or with super/subset queries */
             flecs_record_add_flag(tgt_r, EcsEntityIsTraversable);
 
             /* Add reference to (*, tgt) component record to entity record */
-            tgt_r->cdr = idr_t;
+            tgt_r->cr = cr_t;
         }
 
         /* If second element of pair determines the type, check if the pair 
          * should be stored as a sparse component. */
-        if (cdr->type_info && cdr->type_info->component == tgt) {
+        if (cr->type_info && cr->type_info->component == tgt) {
             if (ecs_has_id(world, tgt, EcsSparse)) {
-                cdr->flags |= EcsIdIsSparse;
+                cr->flags |= EcsIdIsSparse;
             }
+            if (ecs_has_id(world, tgt, EcsDontFragment)) {
+                cr->flags |= EcsIdDontFragment;
+            }
+        }
+
+        /* Check if we should keep a list of ordered children for parent */
+        if (rel == EcsChildOf) {
+            if (ecs_has_id(world, tgt, EcsOrderedChildren)) {
+                cr->flags |= EcsIdOrderedChildren;
+            }
+        }
+
+        /* Mark (*, tgt) record with HasDontFragment so that queries can quickly
+         * detect if there are any non-fragmenting records to consider for a
+         * (*, tgt) query. */
+        if (cr->flags & EcsIdDontFragment && ECS_PAIR_FIRST(id) != EcsFlag) {
+            ecs_assert(cr_t != NULL, ECS_INTERNAL_ERROR, NULL);
+            cr_t->flags |= EcsIdMatchDontFragment;
         }
     }
 
-    cdr->flags |= flecs_component_event_flags(world, id);
+    cr->flags |= flecs_component_event_flags(world, id);
 
-    if (cdr->flags & EcsIdIsSparse) {
-        flecs_component_init_sparse(world, cdr);
-    } else if (cdr->flags & EcsIdIsUnion) {
-        if (ECS_IS_PAIR(id) && ECS_PAIR_SECOND(id) == EcsUnion) {
-            flecs_component_init_sparse(world, cdr);
-        }
+    if (cr->flags & EcsIdIsSparse) {
+        flecs_component_init_sparse(world, cr);
+    }
+
+    if (cr->flags & EcsIdDontFragment) {
+        flecs_component_record_init_dont_fragment(world, cr);
     }
 
     if (ecs_should_log_1()) {
@@ -400,40 +478,40 @@ ecs_component_record_t* flecs_component_new(
     }
 
 #ifdef FLECS_DEBUG_INFO
-    cdr->str = ecs_id_str(world, cdr->id);
+    cr->str = ecs_id_str(world, cr->id);
 #endif
 
     /* Update counters */
     world->info.id_create_total ++;
-    world->info.component_id_count += cdr->type_info != NULL;
-    world->info.tag_id_count += cdr->type_info == NULL;
+    world->info.component_id_count += cr->type_info != NULL;
+    world->info.tag_id_count += cr->type_info == NULL;
     world->info.pair_id_count += is_pair;
 
-    return cdr;
+    return cr;
 }
 
 static
 void flecs_component_assert_empty(
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
-    (void)cdr;
-    ecs_assert(flecs_table_cache_count(&cdr->cache) == 0, 
+    (void)cr;
+    ecs_assert(flecs_table_cache_count(&cr->cache) == 0, 
         ECS_INTERNAL_ERROR, NULL);
 }
 
 static
 void flecs_component_free(
     ecs_world_t *world,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
     flecs_poly_assert(world, ecs_world_t);
-    ecs_assert(cdr != NULL, ECS_INTERNAL_ERROR, NULL);
-    ecs_id_t id = cdr->id;
+    ecs_assert(cr != NULL, ECS_INTERNAL_ERROR, NULL);
+    ecs_id_t id = cr->id;
 
-    flecs_component_assert_empty(cdr);
+    flecs_component_assert_empty(cr);
 
     /* Id is still in use by a query */
-    ecs_assert((world->flags & EcsWorldQuit) || (cdr->keep_alive == 0), 
+    ecs_assert((world->flags & EcsWorldQuit) || (cr->keep_alive == 0), 
         ECS_ID_IN_USE, "cannot delete id that is queried for");
 
     if (ECS_IS_PAIR(id)) {
@@ -442,8 +520,8 @@ void flecs_component_free(
         if (!ecs_id_is_wildcard(id)) {
             if (ECS_PAIR_FIRST(id) != EcsFlag) {
                 /* If id is not a wildcard, remove it from the wildcard lists */
-                flecs_remove_id_elem(cdr, ecs_pair(rel, EcsWildcard));
-                flecs_remove_id_elem(cdr, ecs_pair(EcsWildcard, tgt));
+                flecs_remove_id_elem(cr, ecs_pair(rel, EcsWildcard));
+                flecs_remove_id_elem(cr, ecs_pair(EcsWildcard, tgt));
             }
         } else {
             ecs_log_push_2();
@@ -452,7 +530,7 @@ void flecs_component_free(
              * wildcard are also empty, so release them */
             if (ECS_PAIR_FIRST(id) == EcsWildcard) {
                 /* Iterate (*, Target) list */
-                ecs_component_record_t *cur, *next = cdr->pair->second.next;
+                ecs_component_record_t *cur, *next = cr->pair->second.next;
                 while ((cur = next)) {
                     flecs_component_assert_empty(cur);
                     next = cur->pair->second.next;
@@ -462,7 +540,7 @@ void flecs_component_free(
                 /* Iterate (Relationship, *) list */
                 ecs_assert(ECS_PAIR_SECOND(id) == EcsWildcard, 
                     ECS_INTERNAL_ERROR, NULL);
-                ecs_component_record_t *cur, *next = cdr->pair->first.next;
+                ecs_component_record_t *cur, *next = cr->pair->first.next;
                 while ((cur = next)) {
                     flecs_component_assert_empty(cur);
                     next = cur->pair->first.next;
@@ -475,23 +553,28 @@ void flecs_component_free(
     }
 
     /* Cleanup sparse storage */
-    flecs_component_fini_sparse(world, cdr);
+    flecs_component_fini_sparse(world, cr);
+
+    if (cr->flags & EcsIdDontFragment) {
+        flecs_component_record_fini_dont_fragment(world, cr);
+    }
 
     /* Update counters */
     world->info.id_delete_total ++;
     world->info.pair_id_count -= ECS_IS_PAIR(id);
-    world->info.component_id_count -= cdr->type_info != NULL;
-    world->info.tag_id_count -= cdr->type_info == NULL;
+    world->info.component_id_count -= cr->type_info != NULL;
+    world->info.tag_id_count -= cr->type_info == NULL;
 
     /* Unregister the component record from the world & free resources */
-    ecs_table_cache_fini(&cdr->cache);
+    ecs_table_cache_fini(&cr->cache);
 
-    if (cdr->pair) {
-        flecs_name_index_free(cdr->pair->name_index);
-        ecs_vec_fini_t(&world->allocator, &cdr->pair->reachable.ids, 
+    if (cr->pair) {
+        flecs_ordered_children_fini(world, cr);
+        flecs_name_index_free(cr->pair->name_index);
+        ecs_vec_fini_t(&world->allocator, &cr->pair->reachable.ids, 
             ecs_reachable_elem_t);
         flecs_bfree_w_dbg_info(&world->allocators.pair_id_record, 
-                cdr->pair, "ecs_pair_id_record_t");
+                cr->pair, "ecs_pair_record_t");
     }
 
     ecs_id_t hash = flecs_component_hash(id);
@@ -502,11 +585,11 @@ void flecs_component_free(
     }
 
 #ifdef FLECS_DEBUG_INFO
-    ecs_os_free(cdr->str);
+    ecs_os_free(cr->str);
 #endif
 
     flecs_bfree_w_dbg_info(&world->allocators.id_record, 
-        cdr, "ecs_component_record_t");
+        cr, "ecs_component_record_t");
 
     if (ecs_should_log_1()) {
         char *id_str = ecs_id_str(world, id);
@@ -519,11 +602,11 @@ ecs_component_record_t* flecs_components_ensure(
     ecs_world_t *world,
     ecs_id_t id)
 {
-    ecs_component_record_t *cdr = flecs_components_get(world, id);
-    if (!cdr) {
-        cdr = flecs_component_new(world, id);
+    ecs_component_record_t *cr = flecs_components_get(world, id);
+    if (!cr) {
+        cr = flecs_component_new(world, id);
     }
-    return cdr;
+    return cr;
 }
 
 ecs_component_record_t* flecs_components_get(
@@ -532,41 +615,41 @@ ecs_component_record_t* flecs_components_get(
 {
     flecs_poly_assert(world, ecs_world_t);
     if (id == ecs_pair(EcsIsA, EcsWildcard)) {
-        return world->idr_isa_wildcard;
+        return world->cr_isa_wildcard;
     } else if (id == ecs_pair(EcsChildOf, EcsWildcard)) {
-        return world->idr_childof_wildcard;
+        return world->cr_childof_wildcard;
     } else if (id == ecs_pair_t(EcsIdentifier, EcsName)) {
-        return world->idr_identifier_name;
+        return world->cr_identifier_name;
     }
 
     ecs_id_t hash = flecs_component_hash(id);
-    ecs_component_record_t *cdr = NULL;
+    ecs_component_record_t *cr = NULL;
     if (hash >= FLECS_HI_ID_RECORD_ID) {
-        cdr = ecs_map_get_deref(&world->id_index_hi, ecs_component_record_t, hash);
+        cr = ecs_map_get_deref(&world->id_index_hi, ecs_component_record_t, hash);
     } else {
-        cdr = world->id_index_lo[hash];
+        cr = world->id_index_lo[hash];
     }
 
-    return cdr;
+    return cr;
 }
 
 void flecs_component_claim(
     ecs_world_t *world,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
     (void)world;
-    cdr->refcount ++;
+    cr->refcount ++;
 }
 
 int32_t flecs_component_release(
     ecs_world_t *world,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
-    int32_t rc = -- cdr->refcount;
+    int32_t rc = -- cr->refcount;
     ecs_assert(rc >= 0, ECS_INTERNAL_ERROR, NULL);
 
     if (!rc) {
-        flecs_component_free(world, cdr);
+        flecs_component_free(world, cr);
     }
 
     return rc;
@@ -574,10 +657,10 @@ int32_t flecs_component_release(
 
 void flecs_component_release_tables(
     ecs_world_t *world,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
     ecs_table_cache_iter_t it;
-    if (flecs_table_cache_all_iter(&cdr->cache, &it)) {
+    if (flecs_table_cache_all_iter(&cr->cache, &it)) {
         const ecs_table_record_t *tr;
         while ((tr = flecs_table_cache_next(&it, ecs_table_record_t))) {
             /* Release current table */
@@ -588,38 +671,38 @@ void flecs_component_release_tables(
 
 bool flecs_component_set_type_info(
     ecs_world_t *world,
-    ecs_component_record_t *cdr,
+    ecs_component_record_t *cr,
     const ecs_type_info_t *ti)
 {
-    bool is_wildcard = ecs_id_is_wildcard(cdr->id);
+    bool is_wildcard = ecs_id_is_wildcard(cr->id);
     if (!is_wildcard) {
         if (ti) {
-            if (!cdr->type_info) {
+            if (!cr->type_info) {
                 world->info.tag_id_count --;
                 world->info.component_id_count ++;
             }
         } else {
-            if (cdr->type_info) {
+            if (cr->type_info) {
                 world->info.tag_id_count ++;
                 world->info.component_id_count --;
             }
         }
     }
 
-    bool changed = cdr->type_info != ti;
-    cdr->type_info = ti;
+    bool changed = cr->type_info != ti;
+    cr->type_info = ti;
 
     return changed;
 }
 
 ecs_hashmap_t* flecs_component_name_index_ensure(
     ecs_world_t *world,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
-    ecs_assert(cdr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
-    ecs_hashmap_t *map = cdr->pair->name_index;
+    ecs_assert(cr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
+    ecs_hashmap_t *map = cr->pair->name_index;
     if (!map) {
-        map = cdr->pair->name_index = 
+        map = cr->pair->name_index = 
             flecs_name_index_new(world, &world->allocator);
     }
 
@@ -628,30 +711,30 @@ ecs_hashmap_t* flecs_component_name_index_ensure(
 
 ecs_hashmap_t* flecs_component_name_index_get(
     const ecs_world_t *world,
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
     flecs_poly_assert(world, ecs_world_t);
     (void)world;
-    return cdr->pair->name_index;
+    return cr->pair->name_index;
 }
 
 const ecs_table_record_t* flecs_component_get_table(
-    const ecs_component_record_t *cdr,
+    const ecs_component_record_t *cr,
     const ecs_table_t *table)
 {
-    ecs_assert(cdr != NULL, ECS_INTERNAL_ERROR, NULL);
-    return ecs_table_cache_get(&cdr->cache, table);
+    ecs_assert(cr != NULL, ECS_INTERNAL_ERROR, NULL);
+    return ecs_table_cache_get(&cr->cache, table);
 }
 
 void flecs_components_init(
     ecs_world_t *world)
 {
     /* Cache often used id records on world */
-    world->idr_wildcard = flecs_components_ensure(world, EcsWildcard);
-    world->idr_wildcard_wildcard = flecs_components_ensure(world, 
+    world->cr_wildcard = flecs_components_ensure(world, EcsWildcard);
+    world->cr_wildcard_wildcard = flecs_components_ensure(world, 
         ecs_pair(EcsWildcard, EcsWildcard));
-    world->idr_any = flecs_components_ensure(world, EcsAny);
-    world->idr_isa_wildcard = flecs_components_ensure(world, 
+    world->cr_any = flecs_components_ensure(world, EcsAny);
+    world->cr_isa_wildcard = flecs_components_ensure(world, 
         ecs_pair(EcsIsA, EcsWildcard));
 }
 
@@ -669,9 +752,9 @@ void flecs_components_fini(
 
     int32_t i;
     for (i = 0; i < FLECS_HI_ID_RECORD_ID; i ++) {
-        ecs_component_record_t *cdr = world->id_index_lo[i];
-        if (cdr) {
-            flecs_component_release(world, cdr);
+        ecs_component_record_t *cr = world->id_index_lo[i];
+        if (cr) {
+            flecs_component_release(world, cr);
         }
     }
 
@@ -687,13 +770,13 @@ ecs_flags32_t flecs_id_flags(
     ecs_world_t *world,
     ecs_id_t id)
 {
-    const ecs_component_record_t *cdr = flecs_components_get(world, id);
-    if (cdr) {
+    const ecs_component_record_t *cr = flecs_components_get(world, id);
+    if (cr) {
         ecs_flags32_t extra_flags = 0;
-        if (cdr->flags & EcsIdOnInstantiateInherit) {
+        if (cr->flags & EcsIdOnInstantiateInherit) {
             extra_flags |= EcsIdHasOnAdd|EcsIdHasOnRemove;
         }
-        return cdr->flags|extra_flags;
+        return cr->flags|extra_flags;
     }
     return flecs_component_event_flags(world, id);
 }
@@ -732,32 +815,45 @@ ecs_flags32_t flecs_id_flags_get(
     return result;
 }
 
-ecs_component_record_t* flecs_component_first_next(
-    ecs_component_record_t *cdr)
+void flecs_component_delete_sparse(
+    ecs_world_t *world,
+    ecs_component_record_t *cr)
 {
-    ecs_assert(cdr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
-    return cdr->pair->first.next;
+    ecs_assert(cr != NULL, ECS_INTERNAL_ERROR, NULL);
+    ecs_assert(cr->flags & EcsIdIsSparse, ECS_INTERNAL_ERROR, NULL);
+    int32_t i, count = flecs_sparse_count(cr->sparse);
+    const uint64_t *entities = flecs_sparse_ids(cr->sparse);
+    for (i = 0; i < count; i ++) {
+        ecs_delete(world, entities[i]);
+    }
+}
+
+ecs_component_record_t* flecs_component_first_next(
+    ecs_component_record_t *cr)
+{
+    ecs_assert(cr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
+    return cr->pair->first.next;
 }
 
 ecs_component_record_t* flecs_component_second_next(
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
-    ecs_assert(cdr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
-    return cdr->pair->second.next;
+    ecs_assert(cr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
+    return cr->pair->second.next;
 }
 
 ecs_component_record_t* flecs_component_trav_next(
-    ecs_component_record_t *cdr)
+    ecs_component_record_t *cr)
 {
-    ecs_assert(cdr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
-    return cdr->pair->trav.next;
+    ecs_assert(cr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
+    return cr->pair->trav.next;
 }
 
 bool flecs_component_iter(
-    const ecs_component_record_t *cdr,
+    const ecs_component_record_t *cr,
     ecs_table_cache_iter_t *iter_out)
 {
-    return flecs_table_cache_all_iter(&cdr->cache, iter_out);
+    return flecs_table_cache_all_iter(&cr->cache, iter_out);
 }
 
 const ecs_table_record_t* flecs_component_next(
@@ -765,3 +861,10 @@ const ecs_table_record_t* flecs_component_next(
 {
     return flecs_table_cache_next(iter, ecs_table_record_t);
 }
+
+ecs_id_t flecs_component_get_id(
+    const ecs_component_record_t *cr)
+{
+    ecs_assert(cr != NULL, ECS_INVALID_PARAMETER, NULL);
+    return cr->id;
+}
