@@ -178,8 +178,14 @@ int flecs_type_new_filtered(
     }
 
     dst->count = w;
+
     if (w != count) {
-        dst->array = flecs_wrealloc_n(world, ecs_id_t, w, count, dst->array);
+        if (w) {
+            dst->array = flecs_wrealloc_n(world, ecs_id_t, w, count, dst->array);
+        } else {
+            flecs_wfree_n(world, ecs_id_t, count, dst->array);
+            dst->array = NULL;
+        }
     }
 
     return 0;
@@ -270,7 +276,6 @@ void flecs_type_free(
 }
 
 /* Add to type */
-static
 void flecs_type_add(
     ecs_world_t *world,
     ecs_type_t *type,
@@ -285,6 +290,21 @@ void flecs_type_add(
     }
 }
 
+/* Remove from type */
+void flecs_type_remove(
+    ecs_world_t *world,
+    ecs_type_t *type,
+    ecs_id_t remove)
+{
+    ecs_type_t new_type;
+    int res = flecs_type_new_without(world, &new_type, type, remove);
+    if (res != -1) {
+        flecs_type_free(world, type);
+        type->array = new_type.array;
+        type->count = new_type.count;
+    }
+}
+
 /* Graph edge utilities */
 
 void flecs_table_diff_builder_init(
@@ -292,8 +312,8 @@ void flecs_table_diff_builder_init(
     ecs_table_diff_builder_t *builder)
 {
     ecs_allocator_t *a = &world->allocator;
-    ecs_vec_init_t(a, &builder->added, ecs_id_t, 256);
-    ecs_vec_init_t(a, &builder->removed, ecs_id_t, 256);
+    ecs_vec_init_t(a, &builder->added, ecs_id_t, 32);
+    ecs_vec_init_t(a, &builder->removed, ecs_id_t, 32);
     builder->added_flags = 0;
     builder->removed_flags = 0;
 }
@@ -529,6 +549,8 @@ ecs_table_t *flecs_table_new(
     flecs_hashmap_result_t table_elem,
     ecs_table_t *prev)
 {
+    flecs_check_exclusive_world_access_write(world);
+
     ecs_os_perf_trace_push("flecs.table.create");
 
     ecs_table_t *result = flecs_sparse_add_t(&world->store.tables, ecs_table_t);
@@ -649,27 +671,42 @@ void flecs_compute_table_diff(
     ecs_table_t *node,
     ecs_table_t *next,
     ecs_graph_edge_t *edge,
-    ecs_id_t id)
+    ecs_id_t id,
+    bool is_remove)
 {
     ecs_type_t node_type = node->type;
     ecs_type_t next_type = next->type;
+    bool childof = false;
 
-    if (ECS_IS_PAIR(id)) {
-        ecs_component_record_t *cdr = flecs_components_get(world, ecs_pair(
-            ECS_PAIR_FIRST(id), EcsWildcard));
-        if (cdr->flags & EcsIdIsUnion) {
-            if (node != next) {
-                id = ecs_pair(ECS_PAIR_FIRST(id), EcsUnion);
-            } else {
-                ecs_table_diff_t *diff = flecs_bcalloc(
-                    &world->allocators.table_diff);
-                diff->added.count = 1;
-                diff->added.array = flecs_wdup_n(world, ecs_id_t, 1, &id);
-                diff->added_flags = EcsTableHasUnion;
-                edge->diff = diff;
-                return;
-            }
+    if (ECS_IS_PAIR(id)) { 
+        childof = ECS_PAIR_FIRST(id) ==  EcsChildOf;
+    }
+
+    bool dont_fragment = false;
+    if (id < FLECS_HI_COMPONENT_ID) {
+        dont_fragment = (world->non_trivial[id] & EcsNonTrivialIdNonFragmenting) != 0;
+        if (dont_fragment) {
+            flecs_components_ensure(world, id);
         }
+    } else {
+        ecs_component_record_t *cr = flecs_components_ensure(world, id);
+        dont_fragment = cr->flags & EcsIdDontFragment;
+    }
+
+    if (dont_fragment) {
+        ecs_table_diff_t *diff = flecs_bcalloc(
+            &world->allocators.table_diff);
+        if (is_remove) {
+            diff->removed.count = 1;
+            diff->removed.array = flecs_wdup_n(world, ecs_id_t, 1, &id);
+            diff->removed_flags = EcsTableHasDontFragment|EcsTableHasSparse;
+        } else {
+            diff->added.count = 1;
+            diff->added.array = flecs_wdup_n(world, ecs_id_t, 1, &id);
+            diff->added_flags = EcsTableHasDontFragment|EcsTableHasSparse;
+        }
+        edge->diff = diff;
+        return;
     }
 
     ecs_id_t *ids_node = node_type.array;
@@ -679,7 +716,7 @@ void flecs_compute_table_diff(
     int32_t added_count = 0;
     int32_t removed_count = 0;
     ecs_flags32_t added_flags = 0, removed_flags = 0;
-    bool trivial_edge = !ECS_HAS_RELATION(id, EcsIsA);
+    bool trivial_edge = !ECS_HAS_RELATION(id, EcsIsA) && !childof;
 
     /* First do a scan to see how big the diff is, so we don't have to realloc
      * or alloc more memory than required. */
@@ -750,6 +787,7 @@ void flecs_compute_table_diff(
     for (; i_next < next_count; i_next ++) {
         flecs_diff_insert_added(world, builder, ids_next[i_next]);
     }
+
     for (; i_node < node_count; i_node ++) {
         flecs_diff_insert_removed(world, builder, ids_node[i_node]);
     }
@@ -760,6 +798,15 @@ void flecs_compute_table_diff(
     diff->added_flags = added_flags;
     diff->removed_flags = removed_flags;
 
+    if (ECS_IS_PAIR(id) && ECS_PAIR_FIRST(id) == EcsChildOf) {
+        if (added_count) {
+            diff->added_flags |= EcsTableEdgeReparent;
+        } else {
+            ecs_assert(removed_count != 0, ECS_INTERNAL_ERROR, NULL);
+            diff->removed_flags |= EcsTableEdgeReparent;
+        }
+    }
+
     ecs_assert(diff->added.count == added_count, ECS_INTERNAL_ERROR, NULL);
     ecs_assert(diff->removed.count == removed_count, ECS_INTERNAL_ERROR, NULL);
 }
@@ -787,10 +834,20 @@ void flecs_add_overrides_for_base(
             ecs_id_t to_add = 0;
             if (ECS_HAS_ID_FLAG(id, AUTO_OVERRIDE)) {
                 to_add = id & ~ECS_AUTO_OVERRIDE;
+                ecs_component_record_t *cr = flecs_components_get(
+                    world, to_add);
+                if (cr && (cr->flags & EcsIdDontFragment)) {
+                    to_add = 0;
+
+                    /* Add flag to base table. Cheaper to do here vs adding an
+                     * observer for OnAdd AUTO_OVERRIDE|* / during table 
+                     * creation. */
+                    base_table->flags |= EcsTableOverrideDontFragment;
+                }
             } else {
                 ecs_table_record_t *tr = &base_table->_->records[i];
-                ecs_component_record_t *cdr = (ecs_component_record_t*)tr->hdr.cache;
-                if (ECS_ID_ON_INSTANTIATE(cdr->flags) == EcsOverride) {
+                ecs_component_record_t *cr = (ecs_component_record_t*)tr->hdr.cache;
+                if (ECS_ID_ON_INSTANTIATE(cr->flags) == EcsOverride) {
                     to_add = id;
                 }
             }
@@ -799,9 +856,9 @@ void flecs_add_overrides_for_base(
                 ecs_id_t wc = ecs_pair(ECS_PAIR_FIRST(to_add), EcsWildcard);
                 bool exclusive = false;
                 if (ECS_IS_PAIR(to_add)) {
-                    ecs_component_record_t *cdr = flecs_components_get(world, wc);
-                    if (cdr) {
-                        exclusive = (cdr->flags & EcsIdExclusive) != 0;
+                    ecs_component_record_t *cr = flecs_components_get(world, wc);
+                    if (cr) {
+                        exclusive = (cr->flags & EcsIdExclusive) != 0;
                     }
                 }
                 if (!exclusive) {
@@ -820,7 +877,7 @@ void flecs_add_overrides_for_base(
 
     if (flags & EcsTableHasIsA) {
         const ecs_table_record_t *tr = flecs_component_get_table(
-            world->idr_isa_wildcard, base_table);
+            world->cr_isa_wildcard, base_table);
         ecs_assert(tr != NULL, ECS_INTERNAL_ERROR, NULL);
         int32_t i = tr->index, end = i + tr->count;
         for (; i != end; i ++) {
@@ -832,7 +889,7 @@ void flecs_add_overrides_for_base(
 static
 void flecs_add_with_property(
     ecs_world_t *world,
-    ecs_component_record_t *idr_with_wildcard,
+    ecs_component_record_t *cr_with_wildcard,
     ecs_type_t *dst_type,
     ecs_entity_t r,
     ecs_entity_t o)
@@ -845,7 +902,7 @@ void flecs_add_with_property(
     ecs_assert(table != NULL, ECS_INTERNAL_ERROR, NULL);
     
     const ecs_table_record_t *tr = flecs_component_get_table(
-        idr_with_wildcard, table);
+        cr_with_wildcard, table);
     if (tr) {
         int32_t i = tr->index, end = i + tr->count;
         ecs_id_t *ids = table->type.array;
@@ -860,10 +917,9 @@ void flecs_add_with_property(
             }
 
             flecs_type_add(world, dst_type, a);
-            flecs_add_with_property(world, idr_with_wildcard, dst_type, ra, o);
+            flecs_add_with_property(world, cr_with_wildcard, dst_type, ra, o);
         }
     }
-
 }
 
 static
@@ -874,18 +930,16 @@ ecs_table_t* flecs_find_table_with(
 {
     ecs_make_alive_id(world, with);
 
-    ecs_component_record_t *cdr = NULL;
+    ecs_component_record_t *cr = NULL;
     ecs_entity_t r = 0, o = 0;
 
     if (ECS_IS_PAIR(with)) {
         r = ECS_PAIR_FIRST(with);
         o = ECS_PAIR_SECOND(with);
-        cdr = flecs_components_ensure(world, ecs_pair(r, EcsWildcard));
-        if (cdr->flags & EcsIdIsUnion) {
-            with = ecs_pair(r, EcsUnion);
-        } else if (cdr->flags & EcsIdExclusive) {
+        cr = flecs_components_ensure(world, ecs_pair(r, EcsWildcard));
+        if (cr->flags & EcsIdExclusive) {
             /* Relationship is exclusive, check if table already has it */
-            const ecs_table_record_t *tr = flecs_component_get_table(cdr, node);
+            const ecs_table_record_t *tr = flecs_component_get_table(cr, node);
             if (tr) {
                 /* Table already has an instance of the relationship, create
                  * a new id sequence with the existing id replaced */
@@ -896,10 +950,16 @@ ecs_table_t* flecs_find_table_with(
             }
         }
     } else {
-        cdr = flecs_components_ensure(world, with);
+        cr = flecs_components_ensure(world, with);
         r = with;
     }
 
+    if (cr->flags & EcsIdDontFragment) {
+        /* Component doesn't fragment tables */
+        node->flags |= EcsTableHasDontFragment;
+        return node;
+    }
+
     /* Create sequence with new id */
     ecs_type_t dst_type;
     int res = flecs_type_new_with(world, &dst_type, &node->type, with);
@@ -917,11 +977,11 @@ ecs_table_t* flecs_find_table_with(
         }
     }
 
-    if (cdr->flags & EcsIdWith) {
-        ecs_component_record_t *idr_with_wildcard = flecs_components_get(world,
+    if (cr->flags & EcsIdWith) {
+        ecs_component_record_t *cr_with_wildcard = flecs_components_get(world,
             ecs_pair(EcsWith, EcsWildcard));
         /* If id has With property, add targets to type */
-        flecs_add_with_property(world, idr_with_wildcard, &dst_type, r, o);
+        flecs_add_with_property(world, cr_with_wildcard, &dst_type, r, o);
     }
 
     return flecs_table_ensure(world, &dst_type, true, node);
@@ -933,13 +993,24 @@ ecs_table_t* flecs_find_table_without(
     ecs_table_t *node,
     ecs_id_t without)
 {
+    ecs_component_record_t *cr = NULL;
+
     if (ECS_IS_PAIR(without)) {
-        ecs_entity_t r = 0;
-        ecs_component_record_t *cdr = NULL;
-        r = ECS_PAIR_FIRST(without);
-        cdr = flecs_components_get(world, ecs_pair(r, EcsWildcard));
-        if (cdr && cdr->flags & EcsIdIsUnion) {
-            without = ecs_pair(r, EcsUnion);
+        ecs_entity_t r = ECS_PAIR_FIRST(without);
+        cr = flecs_components_get(world, ecs_pair(r, EcsWildcard));
+        if (cr) {
+            if (cr->flags & EcsIdDontFragment) {
+                node->flags |= EcsTableHasDontFragment;
+                /* Component doesn't fragment tables */
+                return node;
+            }
+        }
+    } else {
+        cr = flecs_components_get(world, without);
+        if (cr && cr->flags & EcsIdDontFragment) {
+            node->flags |= EcsTableHasDontFragment;
+            /* Component doesn't fragment tables */
+            return node;
         }
     }
 
@@ -983,7 +1054,7 @@ void flecs_init_edge_for_add(
 
     flecs_table_ensure_hi_edge(world, &table->node.add, id);
 
-    if ((table != to) || (table->flags & EcsTableHasUnion)) {
+    if ((table != to) || (table->flags & EcsTableHasDontFragment)) {
         /* Add edges are appended to refs.next */
         ecs_graph_edge_hdr_t *to_refs = &to->node.refs;
         ecs_graph_edge_hdr_t *next = to_refs->next;
@@ -996,7 +1067,7 @@ void flecs_init_edge_for_add(
             next->prev = &edge->hdr;
         }
 
-        flecs_compute_table_diff(world, table, to, edge, id);
+        flecs_compute_table_diff(world, table, to, edge, id, false);
     }
 }
 
@@ -1012,7 +1083,7 @@ void flecs_init_edge_for_remove(
 
     flecs_table_ensure_hi_edge(world, &table->node.remove, id);
 
-    if (table != to) {
+    if ((table != to) || (table->flags & EcsTableHasDontFragment)) {
         /* Remove edges are appended to refs.prev */
         ecs_graph_edge_hdr_t *to_refs = &to->node.refs;
         ecs_graph_edge_hdr_t *prev = to_refs->prev;
@@ -1025,7 +1096,7 @@ void flecs_init_edge_for_remove(
             prev->next = &edge->hdr;
         }
 
-        flecs_compute_table_diff(world, table, to, edge, id);
+        flecs_compute_table_diff(world, table, to, edge, id, true);
     }
 }
 
@@ -1076,7 +1147,7 @@ ecs_table_t* flecs_table_traverse_remove(
         ecs_assert(edge->to != NULL, ECS_INTERNAL_ERROR, NULL);
     }
 
-    if (node != to) {
+    if (node != to || edge->diff) {
         if (edge->diff) {
             *diff = *edge->diff;
         } else {
@@ -1118,13 +1189,6 @@ ecs_table_t* flecs_table_traverse_add(
     if (node != to || edge->diff) {
         if (edge->diff) {
             *diff = *edge->diff;
-            if (diff->added_flags & EcsIdIsUnion) {
-                if (diff->added.count == 1) {
-                    diff->added.array = id_ptr;
-                    diff->added.count = 1;
-                    diff->removed.count = 0;
-                }
-            }
         } else {
             diff->added.array = id_ptr;
             diff->added.count = 1;
@@ -1279,6 +1343,21 @@ void flecs_table_clear_edges(
     ecs_log_pop_1();
 }
 
+ecs_table_t* flecs_find_table_add(
+    ecs_world_t *world,
+    ecs_table_t *table,
+    ecs_id_t id,
+    ecs_table_diff_builder_t *diff)
+{
+    ecs_table_diff_t temp_diff = ECS_TABLE_DIFF_INIT;
+    table = flecs_table_traverse_add(world, table, &id, &temp_diff);
+    ecs_check(table != NULL, ECS_INVALID_PARAMETER, NULL);
+    flecs_table_diff_build_append_table(world, diff, &temp_diff);
+    return table;
+error:
+    return NULL;
+}
+
 /* Public convenience functions for traversing table graph */
 ecs_table_t* ecs_table_add_id(
     ecs_world_t *world,
