@@ -1707,15 +1707,15 @@ void Pipeline_stack_allocator_after_progress(void) {
     });
 
     ecs_iter_t it = ecs_query_iter(world, f);
-    ecs_stack_cursor_t cursor = *it.priv_.cache.stack_cursor;
+    ecs_stack_cursor_t cursor = *it.priv_.stack_cursor;
     ecs_iter_fini(&it);
 
     ecs_progress(world, 1);
     test_int(sys_a_invoked, 1);
 
     it = ecs_query_iter(world, f);
-    test_assert(it.priv_.cache.stack_cursor->page == cursor.page);
-    test_assert(it.priv_.cache.stack_cursor->sp == cursor.sp);
+    test_assert(it.priv_.stack_cursor->page == cursor.page);
+    test_assert(it.priv_.stack_cursor->sp == cursor.sp);
     ecs_iter_fini(&it);
 
     ecs_query_fini(f);
@@ -1737,7 +1737,7 @@ void Pipeline_stack_allocator_after_progress_w_pipeline_change(void) {
     });
 
     ecs_iter_t it = ecs_query_iter(world, f);
-    ecs_stack_cursor_t cursor = *it.priv_.cache.stack_cursor;
+    ecs_stack_cursor_t cursor = *it.priv_.stack_cursor;
     ecs_iter_fini(&it);
 
     ecs_progress(world, 1);
@@ -1745,8 +1745,8 @@ void Pipeline_stack_allocator_after_progress_w_pipeline_change(void) {
     test_int(sys_b_invoked, 1);
 
     it = ecs_query_iter(world, f);
-    test_assert(it.priv_.cache.stack_cursor->page == cursor.page);
-    test_assert(it.priv_.cache.stack_cursor->sp == cursor.sp);
+    test_assert(it.priv_.stack_cursor->page == cursor.page);
+    test_assert(it.priv_.stack_cursor->sp == cursor.sp);
     ecs_iter_fini(&it);
 
     ecs_enable(world, SysB, false);
@@ -1756,8 +1756,8 @@ void Pipeline_stack_allocator_after_progress_w_pipeline_change(void) {
     test_int(sys_b_invoked, 1);
 
     it = ecs_query_iter(world, f);
-    test_assert(it.priv_.cache.stack_cursor->page == cursor.page);
-    test_assert(it.priv_.cache.stack_cursor->sp == cursor.sp);
+    test_assert(it.priv_.stack_cursor->page == cursor.page);
+    test_assert(it.priv_.stack_cursor->sp == cursor.sp);
     ecs_iter_fini(&it);
 
     ecs_query_fini(f);
@@ -3336,3 +3336,36 @@ void Pipeline_inout_none_after_write(void) {
 
     ecs_fini(world);
 }
+
+void Pipeline_empty_pipeline_after_disable_phase(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_IMPORT(world, FlecsPipeline);
+
+    ecs_entity_t CustomPhase = ecs_new_w_id(world, EcsPhase);
+
+    ecs_system(world, {
+        .entity = ecs_entity(world, {
+            .add = ecs_ids(ecs_pair(EcsDependsOn, CustomPhase))
+        }),
+        .callback = SysA
+    });
+
+    ecs_enable(world,  CustomPhase, false);
+    ecs_progress(world, 0);
+    test_int(sys_a_invoked, 0);
+
+    ecs_enable(world,  CustomPhase, true);
+    ecs_progress(world, 0);
+    test_int(sys_a_invoked, 1);
+
+    ecs_enable(world,  CustomPhase, false);
+    ecs_progress(world, 0);
+    test_int(sys_a_invoked, 1);
+
+    ecs_enable(world,  CustomPhase, true);
+    ecs_progress(world, 0);
+    test_int(sys_a_invoked, 2);
+
+    ecs_fini(world);
+}
