@@ -205,21 +205,21 @@ void Sparse_remove_all(void) {
 
     flecs_sparse_remove_t(sp, int, 0);
     test_int(flecs_sparse_count(sp), 2);
-    test_assert(flecs_sparse_try_t(sp, int, 0) == NULL);
-    test_assert(flecs_sparse_try_t(sp, int, 1) == elem2);
-    test_assert(flecs_sparse_try_t(sp, int, 2) == elem3);
+    test_assert(flecs_sparse_get_t(sp, int, 0) == NULL);
+    test_assert(flecs_sparse_get_t(sp, int, 1) == elem2);
+    test_assert(flecs_sparse_get_t(sp, int, 2) == elem3);
 
     flecs_sparse_remove_t(sp, int, 1);
     test_int(flecs_sparse_count(sp), 1);
-    test_assert(flecs_sparse_try_t(sp, int, 0) == NULL);
-    test_assert(flecs_sparse_try_t(sp, int, 1) == NULL);
-    test_assert(flecs_sparse_try_t(sp, int, 2) == elem3);    
+    test_assert(flecs_sparse_get_t(sp, int, 0) == NULL);
+    test_assert(flecs_sparse_get_t(sp, int, 1) == NULL);
+    test_assert(flecs_sparse_get_t(sp, int, 2) == elem3);    
 
     flecs_sparse_remove_t(sp, int, 2);
     test_int(flecs_sparse_count(sp), 0);
-    test_assert(flecs_sparse_try_t(sp, int, 0) == NULL);
-    test_assert(flecs_sparse_try_t(sp, int, 1) == NULL);
-    test_assert(flecs_sparse_try_t(sp, int, 2) == NULL);    
+    test_assert(flecs_sparse_get_t(sp, int, 0) == NULL);
+    test_assert(flecs_sparse_get_t(sp, int, 1) == NULL);
+    test_assert(flecs_sparse_get_t(sp, int, 2) == NULL);    
 
     flecs_sparse_free(sp);
 }
@@ -361,10 +361,10 @@ void Sparse_count_of_null(void) {
 void Sparse_try_low_after_ensure_high(void) {
     ecs_sparse_t *sp = flecs_sparse_new(NULL, NULL, int);
 
-    int *ptr_1 = flecs_sparse_ensure_t(sp, int, 5000);
+    int *ptr_1 = flecs_sparse_insert_t(sp, int, 5000);
     test_assert(ptr_1 != NULL);
 
-    int *ptr_2 = flecs_sparse_try_t(sp, int, 100);
+    int *ptr_2 = flecs_sparse_get_t(sp, int, 100);
     test_assert(ptr_2 == NULL);
 
     flecs_sparse_free(sp);
@@ -373,7 +373,7 @@ void Sparse_try_low_after_ensure_high(void) {
 void Sparse_is_alive_low_after_ensure_high(void) {
     ecs_sparse_t *sp = flecs_sparse_new(NULL, NULL, int);
 
-    int *ptr_1 = flecs_sparse_ensure_t(sp, int, 5000);
+    int *ptr_1 = flecs_sparse_insert_t(sp, int, 5000);
     test_assert(ptr_1 != NULL);
 
     bool v = flecs_sparse_is_alive(sp, 100);
@@ -385,57 +385,10 @@ void Sparse_is_alive_low_after_ensure_high(void) {
 void Sparse_remove_low_after_ensure_high(void) {
     ecs_sparse_t *sp = flecs_sparse_new(NULL, NULL, int);
 
-    int *ptr_1 = flecs_sparse_ensure_t(sp, int, 5000);
+    int *ptr_1 = flecs_sparse_insert_t(sp, int, 5000);
     test_assert(ptr_1 != NULL);
 
     flecs_sparse_remove_t(sp, int, 100);
 
     flecs_sparse_free(sp);
 }
-
-void Sparse_ensure_skip_generation(void) {
-    ecs_sparse_t *sp1 = flecs_sparse_new(NULL, NULL, int);
-    ecs_sparse_t *sp2 = flecs_sparse_new(NULL, NULL, int);
-
-    uint64_t id = flecs_sparse_new_id(sp1);
-    flecs_sparse_ensure(sp2, 0, id);
-
-    test_assert(flecs_sparse_is_alive(sp1, id));
-    test_assert(flecs_sparse_is_alive(sp2, id));
-
-    flecs_sparse_remove(sp1, 0, id);
-    flecs_sparse_remove(sp2, 0, id);
-
-    test_assert(!flecs_sparse_is_alive(sp1, id));
-    test_assert(!flecs_sparse_is_alive(sp2, id));
-
-    uint64_t id_2 = flecs_sparse_new_id(sp1);
-    test_assert((uint32_t)id_2 == id);
-
-    test_assert(flecs_sparse_is_alive(sp1, id_2));
-    test_assert(!flecs_sparse_is_alive(sp2, id_2));
-    test_assert(!flecs_sparse_is_alive(sp1, id));
-    test_assert(!flecs_sparse_is_alive(sp2, id));
-
-    flecs_sparse_remove(sp1, 0, id_2);
-
-    test_assert(!flecs_sparse_is_alive(sp1, id_2));
-    test_assert(!flecs_sparse_is_alive(sp2, id_2));
-    test_assert(!flecs_sparse_is_alive(sp1, id));
-    test_assert(!flecs_sparse_is_alive(sp2, id));
-
-    uint64_t id_3 = flecs_sparse_new_id(sp1);
-    test_assert((uint32_t)id_3 == id);
-    flecs_sparse_ensure(sp2, 0, id_3);
-
-    test_assert(flecs_sparse_is_alive(sp1, id_3));
-    test_assert(flecs_sparse_is_alive(sp2, id_3));
-
-    test_assert(!flecs_sparse_is_alive(sp1, id_2));
-    test_assert(!flecs_sparse_is_alive(sp2, id_2));
-    test_assert(!flecs_sparse_is_alive(sp1, id));
-    test_assert(!flecs_sparse_is_alive(sp2, id));
-
-    flecs_sparse_free(sp1);
-    flecs_sparse_free(sp2);
-}
