@@ -144,33 +144,33 @@ void World_multi_world_component(void) {
         .set(Mass{100});
 
     {
-        const Position *p = w1_e.get<Position>();
+        const Position *p = w1_e.try_get<Position>();
         test_assert(p != nullptr);
         test_int(p->x, 10);
         test_int(p->y, 20);
 
-        const Velocity *v = w1_e.get<Velocity>();
+        const Velocity *v = w1_e.try_get<Velocity>();
         test_assert(v != nullptr);
         test_int(v->x, 1);
         test_int(v->y, 2);
 
-        const Mass *m = w1_e.get<Mass>();
+        const Mass *m = w1_e.try_get<Mass>();
         test_assert(m != nullptr);
         test_int(m->value, 100);
     }
 
     {
-        const Position *p = w2_e.get<Position>();
+        const Position *p = w2_e.try_get<Position>();
         test_assert(p != nullptr);
         test_int(p->x, 10);
         test_int(p->y, 20);
 
-        const Velocity *v = w2_e.get<Velocity>();
+        const Velocity *v = w2_e.try_get<Velocity>();
         test_assert(v != nullptr);
         test_int(v->x, 1);
         test_int(v->y, 2);
 
-        const Mass *m = w2_e.get<Mass>();
+        const Mass *m = w2_e.try_get<Mass>();
         test_assert(m != nullptr);
         test_int(m->value, 100);
     }
@@ -369,7 +369,7 @@ void World_register_component_w_reset_in_multithreaded(void) {
 
     test_assert(e.has<Position>());
     test_assert(e.has(pos));
-    const Position *p = e.get<Position>();
+    const Position *p = e.try_get<Position>();
     test_assert(p != nullptr);
     test_int(p->x, 10);
     test_int(p->y, 20);
@@ -396,7 +396,7 @@ void World_register_short_template(void) {
     test_assert(c != 0);
     test_str(c.name(), "Tmp<Test>");
 
-    const EcsComponent *ptr = c.get<flecs::Component>();
+    const EcsComponent *ptr = c.try_get<flecs::Component>();
     test_assert(ptr != NULL);
     test_int(ptr->size, 4);
     test_int(ptr->alignment, 4);
@@ -496,7 +496,7 @@ void World_implicit_register_w_new_world(void) {
 
         auto e = ecs.entity().set<Position>({10, 20});
         test_assert(e.has<Position>());
-        auto *p = e.get<Position>();
+        auto *p = e.try_get<Position>();
         test_assert(p != NULL);
         test_int(p->x, 10);
         test_int(p->y, 20);
@@ -508,7 +508,7 @@ void World_implicit_register_w_new_world(void) {
 
         auto e = ecs.entity().set<Position>({10, 20});
         test_assert(e.has<Position>());
-        auto *p = e.get<Position>();
+        auto *p = e.try_get<Position>();
         test_assert(p != NULL);
         test_int(p->x, 10);
         test_int(p->y, 20);
@@ -1954,12 +1954,12 @@ void World_atfini_w_ctx(void) {
 void World_get_mut_T(void) {
     flecs::world world;
 
-    Position *p = world.get_mut<Position>();
+    Position *p = world.try_get_mut<Position>();
 
     test_assert(p == nullptr);
 
     world.set<Position>({10, 20});
-    p = world.get_mut<Position>();
+    p = world.try_get_mut<Position>();
 
     test_assert(p != nullptr);
     test_int(p->x, 10);
@@ -1971,12 +1971,12 @@ void World_get_mut_R_T(void) {
 
     struct Tgt { };
 
-    Position *p = world.get_mut<Position, Tgt>();
+    Position *p = world.try_get_mut<Position, Tgt>();
     test_assert(p == nullptr);
 
     world.set<Position, Tgt>({10, 20});
     
-    p = world.get_mut<Position, Tgt>();
+    p = world.try_get_mut<Position, Tgt>();
     test_assert(p != nullptr);
 
     test_int(p->x, 10);
@@ -2056,7 +2056,7 @@ void World_fini_copy_move_assign(void) {
     *finished = true;
     };
 
-    auto get_world_id = [](const flecs::world &world) { return world.get<ID>()->id; };
+    auto get_world_id = [](const flecs::world &world) { return world.try_get<ID>()->id; };
 
     bool finished_1 = false;
     bool finished_2 = false;
@@ -2125,3 +2125,38 @@ void World_world_init_fini_log_all(void) {
 
     test_assert(true);
 }
+
+void World_exclusive_access_self_mutate(void) {
+    flecs::world ecs;
+
+    ecs.exclusive_access_begin();
+
+    flecs::entity e = ecs.entity();
+
+    e.add<Position>();
+    test_assert(e.has<Position>());
+
+    ecs.exclusive_access_end();
+}
+
+void* thread_exclusive_access_other_fini(void *arg) {
+    flecs::world *world = static_cast<flecs::world*>(arg);
+    test_expect_abort();
+    world->entity();
+    return NULL;
+}
+
+void World_exclusive_access_other_mutate(void) {
+    install_test_abort();
+
+    flecs::world ecs;
+
+    ecs.exclusive_access_begin();
+
+    ecs_os_thread_t thr = 
+        ecs_os_thread_new(thread_exclusive_access_other_fini, &ecs);
+
+    ecs_os_thread_join(thr);
+
+    test_assert(false); // should not get here
+}
