@@ -3,7 +3,7 @@
 int main(int argc, char *argv[]) {
     ecs_world_t *world = ecs_init_w_args(argc, argv);
 
-    ECS_IMPORT(world, FlecsMonitor);
+    ECS_IMPORT(world, FlecsStats);
 
     return ecs_fini(world);
 }
