@@ -1605,9 +1605,11 @@ void SerializeEntityToJson_serialize_id_recycled(void) {
 }
 
 void SerializeEntityToJson_serialize_union_target(void) {
+    test_quarantine("25 Jun 2025");
+
     ecs_world_t *world = ecs_init();
 
-    ECS_ENTITY(world, Rel, Union);
+    ECS_ENTITY(world, Rel, DontFragment, Exclusive);
     ECS_TAG(world, TgtA);
     ECS_TAG(world, TgtB);
     ECS_TAG(world, TgtC);
@@ -1642,9 +1644,11 @@ void SerializeEntityToJson_serialize_union_target(void) {
 }
 
 void SerializeEntityToJson_serialize_union_target_recycled(void) {
+    test_quarantine("25 Jun 2025");
+
     ecs_world_t *world = ecs_init();
 
-    ECS_ENTITY(world, Rel, Union);
+    ECS_ENTITY(world, Rel, DontFragment, Exclusive);
 
     ecs_entity_t e = ecs_new(world);
     ecs_delete(world, e);
@@ -2143,3 +2147,22 @@ void SerializeEntityToJson_serialize_toggle_pair(void) {
 
     ecs_fini(world);
 }
+
+void SerializeEntityToJson_serialize_null_doc_name(void) {
+    ecs_world_t *world = ecs_init();
+
+    ecs_entity_t e = ecs_new(world);
+    ecs_set_pair(world, e, EcsDocDescription, EcsName, { NULL });
+
+    char *json = ecs_entity_to_json(world, e, NULL);
+    test_assert(json != NULL);
+    
+    char *expect = flecs_asprintf(
+        "{\"name\":\"#%u\", \"components\":{\"(flecs.doc.Description,flecs.core.Name)\":{\"value\":null}}}", (uint32_t)e);
+
+    test_json(json, expect);
+    ecs_os_free(json);
+    ecs_os_free(expect);
+
+    ecs_fini(world);
+}
