@@ -1301,3 +1301,29 @@ void SerializeToJson_vector_array_i32_3(void) {
 
     ecs_fini(world);
 }
+
+void SerializeToJson_serialize_from_stage(void) {
+    typedef struct {
+        ecs_char_t x;
+    } T;
+
+    ecs_world_t *world = ecs_init();
+
+    ecs_entity_t t = ecs_struct_init(world, &(ecs_struct_desc_t){
+        .entity = ecs_entity(world, {.name = "T"}),
+        .members = {
+            {"x", ecs_id(ecs_char_t)}
+        }
+    });
+
+    T value = {'a'};
+
+    ecs_world_t *stage = ecs_get_stage(world, 0);
+
+    char *expr = ecs_ptr_to_json(stage, t, &value);
+    test_assert(expr != NULL);
+    test_str(expr, "{\"x\":\"a\"}");
+    ecs_os_free(expr);
+
+    ecs_fini(world);
+}
